Package: BiocStyle
Title: Standard styles for vignettes and other Bioconductor documents
Description: Provides standard formatting styles for Bioconductor PDF
    and HTML documents. Package vignettes illustrate use and
    functionality.
Version: 2.40.0
Authors@R: c(
        person("Andrzej", "Oleś", role = "aut",
            comment = c(ORCID = "0000-0003-0285-2787")
        ), 
        person("Mike", "Smith", role = "ctb",
            comment = c(ORCID = "0000-0002-7800-3848")
        ),
        person("Martin", "Morgan", role = "ctb"),
        person("Wolfgang", "Huber", role = "ctb"),
        person("Bioconductor Package Maintainer", role = "cre",
            email = "maintainer@bioconductor.org")
        )
Imports: bookdown, knitr (>= 1.30), rmarkdown (>= 1.2), stats, utils,
        yaml, BiocManager
Suggests: BiocGenerics, RUnit, htmltools
biocViews: Software
License: Artistic-2.0
VignetteBuilder: knitr
Encoding: UTF-8
URL: https://github.com/Bioconductor/BiocStyle
BugReports: https://github.com/Bioconductor/BiocStyle/issues
RoxygenNote: 6.1.0
Date: 2025-08-01
git_url: https://git.bioconductor.org/packages/BiocStyle
git_branch: RELEASE_3_23
git_last_commit: 2817006
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 20:53:47 UTC; biocbuild
Author: Andrzej Oleś [aut] (ORCID: <https://orcid.org/0000-0003-0285-2787>),
  Mike Smith [ctb] (ORCID: <https://orcid.org/0000-0002-7800-3848>),
  Martin Morgan [ctb],
  Wolfgang Huber [ctb],
  Bioconductor Package Maintainer [cre]
Maintainer: Bioconductor Package Maintainer <maintainer@bioconductor.org>

# Generated by roxygen2: do not edit by hand

export(Biocannopkg)
export(Biocbook)
export(Biocexptpkg)
export(Biocpkg)
export(CRANpkg)
export(Githubpkg)
export(Rpackage)
export(doc_date)
export(html_document)
export(latex)
export(markdown)
export(md_document)
export(output)
export(pdf_document)
export(pkg_ver)
export(use_vignette_html)
export(use_vignette_pdf)
importFrom(BiocManager,version)
importFrom(knitr,current_input)
importFrom(knitr,hook_pdfcrop)
importFrom(knitr,hook_plot_md)
importFrom(knitr,hook_plot_tex)
importFrom(knitr,knit_hooks)
importFrom(knitr,knit_theme)
importFrom(knitr,opts_chunk)
importFrom(knitr,opts_hooks)
importFrom(knitr,opts_knit)
importFrom(knitr,set_header)
importFrom(rmarkdown,draft)
importFrom(rmarkdown,knitr_options)
importFrom(rmarkdown,metadata)
importFrom(rmarkdown,output_format)
importFrom(rmarkdown,pandoc_available)
importFrom(rmarkdown,pandoc_version)
importFrom(stats,setNames)
importFrom(utils,packageVersion)
importFrom(yaml,yaml.load)

## create an example HTML file with a footnote to be converted
input <- "This is a test sentence^[here is a footnote]"
tf_md <- tempfile(fileext = ".md")
writeLines(input, con = tf_md)
tf_html <- tempfile(fileext = ".html")
rmarkdown::pandoc_convert(input = tf_md, to = "html", output = tf_html, options = c("--wrap=preserve"))

## convert footnotes to sidenotes
lines <- BiocStyle:::process_footnotes(BiocStyle:::readUTF8(tf_html))

checkTrue(
  any(grepl("sidenote", lines)), msg = "Sidenote not found in HTML output"
)
checkTrue(
  !any(grepl("footnote-ref", lines)), msg = "Footnote found in HTML output"
)

#' Use Bioconductor style to format R Markdown PDF output
#' 
#' This function sets the Bioconductor style in PDF documents rendered using R
#' Markdown v2.
#' 
#' @inheritParams html_document
#' 
#' @param includes Named list of additional content to include within the
#' document (typically created using the \code{\link{includes}} function).
#' @param \dots Additional arguments passed to
#' \code{\link[rmarkdown]{pdf_document}}.
#' @param toc_newpage logical(1), \code{TRUE} to start the table of contents on
#' a new page.
#' @param use_unsrturl logical(1), indicating that the \sQuote{unsrturl} style
#' will be used (\code{\\bibliographystyle} command \emph{not} required).
#' @param relative_path logical(1), copy supporting LaTeX files and use
#' relative paths rather than absolute paths to system files.
#' @return R Markdown output format to pass to \code{\link[rmarkdown]{render}}.
#' @author Andrzej Oleś <andrzej.oles@@embl.de>, 2014-2017
#' @seealso \code{\link[BiocStyle]{html_document}},
#' \code{\link[BiocStyle]{md_document}}
#' @keywords manip
#' @examples
#' 
#' \dontrun{
#' 
#' # simple invocation
#' render("input.Rmd", BiocStyle::pdf_document())
#' 
#' # specify an option for latex engine
#' render("input.Rmd", BiocStyle::pdf_document(toc = FALSE))
#' }
#' 
#' @export
pdf_document <- function(toc = TRUE,
                         number_sections = TRUE,
                         fig_width = NA,
                         fig_height = NA,
                         includes = NULL,
                         ...,
                         ## BiocStyle specific arguments:
                         titlecaps = TRUE,
                         toc_newpage = FALSE,
                         use_unsrturl = TRUE,
                         relative_path = FALSE) {
  
  ## load the package to expose macros
  require(BiocStyle, quietly = TRUE)
  
  relative_path <- isTRUE(relative_path)
  use_unsrturl <- isTRUE(use_unsrturl)
  
  sty <- bioconductor.sty
  bst <- if (use_unsrturl) file.path(resources, "tex", "unsrturl.bst") else NULL
  
  template_files <- c(sty=sty, bst=bst)
  
  pre_knit <- intermediates_generator <- NULL
  
  if ( relative_path ) {
    template_files <- copyResource(template_files, getwd())

    # use `pre_knit` to copy template files as this seems to be the only
    # function which is aware of the original input file name passed to `render`
    pre_knit <- function(input, ...) {
      file.copy(template_files, dirname(normalizePath(input)))
    }

    # run when document is rendered to a different directory than the input file
    intermediates_generator <- function(original_input, encoding, intermediates_dir) {
      file.copy(template_files, normalizePath(intermediates_dir))
      basename(template_files)
    }

  }
  
  template <- create_latex_template(if (isTRUE(titlecaps)) NULL else "notitlecaps", template_files[["sty"]])
  
  head = NULL
  
  ## code chunks and code highlighting
  thm <- system.file("themes", "default.css", package = "BiocStyle")
  opts_knit$set(out.format="latex");
  head = c(head,
           "% code highlighting",
           knit_theme$get(thm)$highlight,
           readLines(file.path(resources, "tex", "highlighting-macros.def")))
  
  if (use_unsrturl)
    head = c(head, sprintf("\\AtBeginDocument{\\bibliographystyle{%s}}\n",  sub(".bst$", "", template_files[["bst"]])))
  
  # dump to a header file which will be included in the template
  header = tempfile("", fileext = ".tex")
  writeLines(head, header)
  
  inc = rmarkdown::includes(in_header = header)
  
  if ( is.null(includes) )
    includes = inc
  else
    includes$in_header = c(includes$in_header, inc$in_header)
  
  # disable pandoc (sub)paragraph adjustment 
  # https://github.com/Bioconductor/BiocStyle/issues/63
  pandoc_args = c("--variable", "subparagraph")
  
  if (isTRUE(toc_newpage))
    pandoc_args = c(pandoc_args, "--variable", "toc-newpage")
  
  # knitr options
  knitr = merge_lists(.knitr_options(), list(
    opts_knit = list(
    ),
    opts_chunk = list(
      collapse = TRUE, R.options = list(width = 80L)
    ),
    knit_hooks = list(
      plot = function(x, options = list()) {
        adjustwidth = NULL
        
        # adjust width for plots inserted not as floats
        if (!length(options$fig.cap) || is.na(options$fig.cap)) {
          adjustwidth = c('\\begin{adjustwidth}{\\fltoffset}{0mm}',
                          '\\end{adjustwidth}')
        }
        
        if (options$fig.env=="figure" && is.null(adjustwidth)) {
          hook_plot_md(x, options)
        }
        else {
          paste0(adjustwidth[1L],
                 hook_plot_tex(x, options),
                 adjustwidth[2L])
        }
      }
    ),
    opts_hooks = list(
      # respect fig.pos, see https://github.com/rstudio/rmarkdown/issues/1012
      fig.pos = function(options) {
        if (is.null(options$out.extra)) {
          options$out.extra = ''
        }
        options
      }
    )
  ))
  
  post_processor = function(metadata, input, output, clean, verbose) {
    lines = readUTF8(output)
    
    ## insert author affiliations
    lines <- modifyLines(lines, from='%% AUTH AFFIL %%', insert=auth_affil_latex(metadata))
    
    ## LaTeX soul hacks
    r = "(?<=\\\\texttt{)((?:\\\\{|\\\\}|[^{}]|{(?1)})*)(?=})"
    m <- gregexpr(r, lines, perl=TRUE)
    regmatches(lines, m) <- lapply(regmatches(lines, m), function(x) {
      # substitute all control spaces "\ " in \texttt by regular spaces
      x <- gsub("\\ ", " ", x, fixed=TRUE)
    })
    
    writeUTF8(lines, output)
    
    output
  }
  
  config <- output_format(
    knitr = knitr,
    pandoc = list(args = pandoc_args),
    pre_knit = pre_knit,
    pre_processor = pre_processor,
    post_processor = post_processor,
    intermediates_generator = intermediates_generator,
    # use bookdown::pdf_book for the added capability of cross-referencing
    base_format = bookdown::pdf_book(
      toc = toc,
      number_sections = number_sections,
      fig_width = fig_width,
      fig_height = fig_height,
      template = "default",
      includes = includes,
      base_format = rmarkdown::pdf_document,
      ...)
  )
  
  ## override the default document template which is used in order to retain
  ## some original template-dependent rmarkdown functionality
  pos <- match("--template", config$pandoc$args)
  if (is.na(pos)) {
    config$pandoc$args <- c(config$pandoc$args, "--template")
    pos <- length(config$pandoc$args)
  }
  config$pandoc$args[ pos + 1L ] <- template
  
  ## do not include subtitle code as the template already provides it
  pos <- grep("*subtitle\\.tex$", config$pandoc$args)
  if (length(pos)==1L)
    config$pandoc$args <- config$pandoc$args[-c(pos-1, pos)]
  
  ## remove the obsolete default 'geometry' pandoc variable after it has beed
  ## added by the default 'pdf_pre_processor' defined in rmarkdown::pdf_document
  pre = config$pre_processor
  config$pre_processor = function(metadata, input_file, runtime, knit_meta, files_dir, output_dir) {
    args = if ( is.function(pre) )
      pre(metadata, input_file, runtime, knit_meta, files_dir, output_dir)
    else
      pre
    
    pos = match("geometry:margin=1in", args)
    if (!is.na(pos))
      args = args[-c(pos-1L, pos)]
    
    # use titling package to change title format to be more compact by default
    pos <- grep("*compact-title\\.tex$", args)
    if (length(pos)==0L)
      pos = length(args) + 1L
    args[c(pos-1, pos)] <- c("--variable", "compact-title:yes")
    
    args
  }
  
  config
}

create_latex_template <- function(opts=NULL, sty=bioconductor.sty) {
  ## get and modify the default pandoc template
  #lines <- system2(rmarkdown::pandoc_exec(), "--print-default-template latex", stdout = TRUE, stderr = TRUE)
  
  # choose the rmarkdown template depending on pandoc version
  version <- pandoc_version()
  
  template_files <- list.files(system.file("rmd", "latex", package="BiocStyle"),
                               pattern = "\\.tex$")
  template_versions <- sub("default-?([0-9.]*).tex", "\\1", template_files)
  template_versions <- numeric_version(template_versions, strict = FALSE)
  template_versions <- sort(template_versions, decreasing = TRUE)
  
  idx <- match(TRUE, version >= template_versions, nomatch = 0L)
  
  template <- if (idx > 0L) sprintf("default-%s.tex", template_versions[idx]) else "default.tex"
  
  # customize the template
  lines <- readUTF8(system.file("rmd", "latex", template, package = "BiocStyle"))
  
  template <- biocTempfile("template.tex")
  
  # remove redundand hyperref definitions
  lines <- modifyLines(lines, from="\\usepackage[unicode=true]{hyperref}", to="\\urlstyle{same}", offset=c(-5L, 0L))
  lines <- modifyLines(lines, from="\\usepackage{hyperref}", to="\\urlstyle{same}") # >= 1.15.2
  
  # do not set links in TOC to black
  lines <- modifyLines(lines, from="\\hypersetup{linkcolor=black}")
  lines <- modifyLines(lines, from="\\hypersetup{linkcolor=$if(toccolor)$$toccolor$$else$black$endif$}") # >= 1.14
  
  # load BiocStyle after 'titling' to override title page formating, but before 
  # author specification to ensure 'hyperref' gets loaded before 'authblk'
  lines <- modifyLines(lines, from="\\title{", replace=FALSE, before=TRUE, insert=c(
    loadBioconductorStyleFile(sty, opts),
    ""))
  
  # use \bioctitle to capture short title for page headings
  lines <- modifyLines(lines, from="\\title{$title$}", insert="\\bioctitle[$if(shorttitle)$$shorttitle$$endif$]{$title$}")
  
  # add author affiliations
  lines <- modifyLines(lines, from="\\author{$for(author)$$author$$sep$ \\\\ $endfor$}", insert="%% AUTH AFFIL %%")
  
  # add package version number
  lines <- modifyLines(lines, from="\\end{abstract}", offset=1L, replace=FALSE, insert=c(
    "",
    "$if(package)$",
    "\\packageVersion{$package$}",
    "$endif$"))
  
  # remove highlighting-macros
  lines <- modifyLines(lines, from="$highlighting-macros$", offset=c(-1L, 1L))
  
  ## output TOC on a separate page
  lines <- modifyLines(lines, from="\\tableofcontents", insert=c(
    "$if(toc-newpage)$",
    "\\newpage",
    "$endif$",
    "\\tableofcontents",
    "\\newpage"))
  
  writeUTF8(lines, template)
  
  template
}

#' Use Bioconductor style to format LaTeX vignettes
#' 
#' 
#' This function inserts code into a document preamble to provide a consistent
#' style to Bioconductor vignettes.
#' 
#' Use is described in the \sQuote{Bioconductor LaTeX Style 2.0} vignette.
#' 
#' By default the 'unsrturl' bibliography style is used, which automatically
#' creates links to URL, DOIs, etc. Use a different \code{bibliographystyle}
#' with \code{use.unsrturl=FALSE} and standard LaTeX commands for styling
#' bibliographies.
#' 
#' @param \dots Additional arguments, passed to \code{\link{options}}.
#' @param width integer(1), maximum number of columns on a line used in
#' printing. See \code{\link{options}}.
#' @param titlecaps logical(1), emphasize the first sentence of float captions
#' @param short.fignames logical(1), indicates whether \code{\\incfig} figures
#' should be inserted and referred to using short names equal to corresponding
#' code chunk labels without any additional prefix.
#' @param fig.path character(1), custom prefix to be used for figure file names
#' when used with \code{knitr}; has no effect when compiled with \code{Sweave}.
#' For details see \code{\link[knitr]{opts_chunk}}.
#' @param use.unsrturl logical(1), indicating that the \sQuote{unsrturl} style
#' will be used (\code{\\bibliographystyle} command \emph{not} required).
#' @param relative.path logical(1), copy supporting LaTeX files and use
#' relative paths rather than absolute paths to system files.
#' @return
#' 
#' The return value is a character string directing LaTex to use the
#' Bioconductor style file.
#' 
#' A side effect is setting any options specified as arguments.
#' @author Andrzej Oleś, Martin Morgan, Wolfgang Huber
#' @keywords manip
#' @examples
#' 
#' ## location of the .sty file
#' BiocStyle:::bioconductor.sty
#' 
#' @export latex
latex <- function(..., width, titlecaps = TRUE, short.fignames=FALSE, fig.path,
                   use.unsrturl=TRUE, relative.path = FALSE) {
  
  sty <- bioconductor.sty
  if ( isTRUE(relative.path) )
    sty <- copyResource(sty, getwd())
      
  cat(loadBioconductorStyleFile(sty, if (isTRUE(titlecaps)) NULL else "notitlecaps"), sep="\n")
  
  if ( isTRUE(use.unsrturl) ) {
    bst <- file.path(resources, "tex", "unsrturl.bst")
    if ( isTRUE(relative.path) )
      bst <- copyResource(bst, getwd())
    cat(sprintf("\\AtBeginDocument{\\bibliographystyle{%s}}", sub(".bst$", "", bst)), sep="\n")
  }
 
    setPrefix = function(x) {
      cat(sprintf("\\renewcommand{\\prefix}[1]{%s#1}", x))
    }
    
    ## check whether called from knitr
    fs = sapply(sys.calls(), function(x) as.character(x)[1])
    knitr = any(grepl("knit", fs, fixed = TRUE))
    
    ## knitr
    if (knitr) {
        if (missing(fig.path)) {
          fig.path = opts_chunk$get("fig.path", default=TRUE)
        } else {
          opts_chunk$set(fig.path=fig.path)
        }
        
        if ( isTRUE(short.fignames) ) setPrefix(fig.path)
      
        ## set knitr options
        opts_knit$set(latex.options.color="usenames,dvipsnames")
        
        opts_chunk$set(
          c(.opts_chunk,
            background = NA, # suppress \definecolor{shadecolor} in knitrout environment 
            fig.width = NA, # reset figure dimensions to detect values set by user
            fig.height = NA)
        )
        
        opts_hooks$set(.opts_hooks)
        
        # set hooks for special plot output
        knit_hooks$set(
          plot = function(x, options = list()) {
            adjustwidth1 = adjustwidth2 = NULL
            
            # adjust width of plots not inserted as floats
            if (!length(options$fig.cap) || is.na(options$fig.cap)) {
              # multiple plots: begin at 1, end at fig.num
              fig.cur = max(options$fig.cur, 1L)
              fig.num = max(options$fig.num, 1L)
              standalone = options$fig.show != 'hold'
              
              # open adjustwidth env if this plot is standalone or first in set
              if (standalone || fig.cur==1L) {
                offsets = switch(options$fig.env, '{\\fltoffset}{0mm}',# default
                                 "smallfigure" = '{2\\fltoffset}{2\\fltoffset}',
                                 "figure*" = '{0mm}{-\\fltoffset}')
                adjustwidth1 = paste0('\\begin{adjustwidth}', offsets)
              }
              # close adjustwidth env if this plot is standalone or last in set
              if (standalone || fig.cur==fig.num)
                adjustwidth2 = '\\end{adjustwidth}'
            }
            
            # call the default knitr hook as defined in render_latex()
            paste0('\\end{kframe}',
                   adjustwidth1,
                   hook_plot_tex(x, options),
                   adjustwidth2,
                   '\\begin{kframe}')
          },
          
          ## remove figure margins with pdfcrop
          crop = hook_pdfcrop,
          
          ## proper aspect ratio of plots
          eval = function(before, options) {
            if (before) {
              #par(mar=c(4,4,0.5,0.5))
            }
          }
        )
        
        ## code highlighting
        thm <- system.file("themes", "default.css", package = "BiocStyle")
        knit_theme$set(thm)
        
        ## reset header$framed which by default includes the content of knitr.sty
        set_header(framed = "\\newenvironment{knitrout}{}{} % an empty environment to be redefined in TeX")
    }
    
    ## assume Sweave
    else {
      if ( isTRUE(short.fignames) ) setPrefix("\\jobname-")
      cat("\\usepackage[noae, nogin]{Sweave}")
    }
    
    ## line width of output code chunks
    if (missing(width)) {
      fontsize = NULL
      
      # try to resolve the optimal width based on document font size setting
      src = if (knitr) {
        current_input()
      } else {
        tryCatch(dynGet("file"), error = function(e) NULL) # TODO: might need to be more specific here
      }
      
      if (!is.null(src)) {
        fontsize = tryCatch({
          lines = readLines(src, warn=FALSE)
          documentclass = grep("^[:blank:]*\\\\documentclass", lines, value = TRUE)[1L]
          sub(".+(1?[01289]pt).+","\\1", documentclass)
        }, error = function(e) NULL)
      }
      
      width = .width(fontsize, knitr)
    }
    
    options(..., width = width)
    }

.width = function(fontsize = "10pt", knitr = TRUE, default = 80L) {
  w = if (is.null(fontsize)) 
    default
  else
    switch(fontsize, default,
           "12pt" = 66L,
           "11pt" = 73L,
           "10pt" = 80L,
           "9pt"  = 89L,
           "8pt"  = 100L)
  
  # knitr output is usually commented out
  if (knitr) {
    com = opts_chunk$get("comment")
    if (!is.null(com) && !is.na(com) && nzchar(com))
      w = w - (nchar(com) + 1L)
  }
  
  w
}

resources <- bioconductor.sty <- bioconductor.css <- NULL

# resolve paths once during package load
.onLoad <- function(...) {
    resources <<- system.file(package = "BiocStyle", "resources")
    bioconductor.sty <<- file.path(resources, "tex", "Bioconductor.sty")
    bioconductor.css <<- file.path(resources, "html", "bioconductor.css")
}

parse_auth_affil = function(m) {
  auth <- m$author
  
  ## missing author metadata
  if ( is.null(auth) )
    return()
  
  res <- list(
    names = NULL,
    affil = NULL,
    affil_marks = NULL,
    email = NULL,
    email_marks = NULL
  )
  
  ## author list with additional metadata
  if (is.list(auth)) {
    ## multiple authors
    if (is.null(names(auth))) {
      ## convert plain entries to named lists for the sake of easier processing
      idx <- !sapply(auth, is.list)
      auth[idx] <- lapply(auth[idx], function(x) list(name = x))
      
      names <- sapply(auth, .subset2, "name")
      email <- sapply(auth, .subset2, "email")
      affil <- lapply(auth, .subset2, "affiliation")
      
      ## author names
      res$names <- names
      
      ## assign IDs to unique affiliations
      affil_uniq <- unique(unlist(affil))
      
      if ( !is.null(affil_uniq) ) {
        affil_ids <- setNames(seq_along(affil_uniq), affil_uniq)
        affil_marks <- sapply(affil, function(a) paste(sort(affil_ids[a]), collapse = ',') )
        
        res$affil <- affil_uniq
        res$affil_marks <- affil_marks
      }
      
      ## assign IDs to emails
      email_uniq <- unique(unlist(email))
      
      if ( !is.null(email_uniq) ) {
        email_ids <- setNames(sapply(seq_along(email_uniq), function(i) paste(rep("*", i), collapse="")), email_uniq)
        email_marks <- sapply(email, function(e) paste(sort(email_ids[e]), collapse = ','))
        
        res$email <- email_uniq
        res$email_marks <- email_marks
      }
    }
    ## single author
    else {
      res$names <- auth$name
      res$email <- auth$email
      res$affil <- auth$affiliation
    }
  }
  ## plain author field
  else {
    res$names = auth
  }
  
  res
}

auth_affil_html = function(m) {
  m <- parse_auth_affil(m)
  
  ## missing author metadata
  if ( is.null(m) )
    return()
  
  ## construct author list
  authors <-
  if ( is.null(m$affil_marks) && is.null(m$email_marks) ) {
    m$names
  } else {
    marks <- paste0(m$affil_marks, m$email_marks)
    idx <- marks != ""
    marks[idx] <- sprintf('<span class="affil-mark">%s</span>', marks[idx])
    paste0(m$names, marks)
  }

  if ((l = length(authors)) > 2)
    authors <- c(paste(authors[1:(l-1L)], collapse=", "), authors[l])
 
  authors <- paste(authors, collapse=" and ")
 
  res <- sprintf('<p class="author-name">%s</p>', authors)
  
  ## construct affiliations list
  if ( !is.null(m$affil) ) {
    if ( !is.null(m$affil_marks) ) {
      affiliations <- sprintf('<span class="affil-mark">%s</span>%s', seq_along(m$affil), m$affil)
    } else {
      affiliations <- m$affil
    }
    affiliations <- paste(affiliations, collapse="<br>")
    res <- c(res, sprintf('<p class="author-affiliation">%s</p>', affiliations))
  }
  
  ## construct email list
  if ( !is.null(m$email) ) {
    emails <- sprintf('<a href="mailto:%s">%s</a>', m$email, m$email)
    if ( !is.null(m$email_marks) ) {
      emails <- sprintf('<span class="affil-mark">%s</span>%s', m$email_marks[m$email_marks!=""], emails)
      emails <- paste(emails, collapse="<br>")
    }
    res <- c(res, sprintf('<p class="author-email">%s</p>', emails))
  }
  
  res
}

auth_affil_latex = function(m) {
  m <- parse_auth_affil(m)
  
  ## missing author metadata
  if ( is.null(m) )
    return()
  
  author <- m$names
  
  ## append emails in \thanks
  if ( !is.null(m$email_marks) ) {
    idx <- m$email_marks != ""
    author[idx] <-  sprintf("%s\\thanks{\\ttfamily %s}", author[idx], m$email)
  }
  
  if ( is.null(m$affil_marks) ) {
    if ((l = length(author)) > 2)
      author <- c(paste(author[1:(l-1L)], collapse=", "), author[l])
    
    author <- paste(author, collapse=" and ")
    author <- sprintf("\\author{%s}", author)
    affil <- NULL
  }
  else {
    author <- sprintf("\\author[%s]{%s}", m$affil_marks, author)
    affil <- sprintf("\\affil[%s]{%s}", seq_along(m$affil), m$affil)
  }
  
  c(author, affil)
}

html_fragment <- function(...,
                          toc = TRUE,
                          number_sections = TRUE,
                          fig_width = 5.67,
                          fig_height = fig_width,
                          fig_align = "right",
                          fig_crop = TRUE,
                          theme = NULL,
                          highlight = NULL,
                          navlinks = TRUE,
                          navlinks_top = TRUE,
                          navlinks_level = 3) {
  
  ## enable BiocStyle macros
  require(BiocStyle, quietly = TRUE)
  
  knitr <- knitr_options(opts_knit = list(width = 80L),
                         # remove figure margins
                         opts_chunk = list(crop = isTRUE(fig_crop),
                                           fig.align = fig_align),
                         knit_hooks = list(crop = hook_pdfcrop))
  
  post_processor <- function(metadata, input, output, clean, verbose) {
    lines <- readUTF8(output)
    
    ## add author affiliations
    lines <- modifyLines(lines, from='<!-- AUTH AFFIL -->', insert=auth_affil_html(metadata))
    
    ## append "References" section header
    if ( !is.null(metadata$bibliography) )
      lines <- add_refs_header(lines)
    
    ## move all headers one level down (for proper formatting when embedded in the website)
    lines <- process_headers(lines)
    
    writeUTF8(lines, output)
    output
  }
  
  config <- output_format(
    knitr = knitr,
    pandoc = NULL,
    post_processor = post_processor,
    base_format = bookdown::html_document2(
      toc = toc,
      number_sections = number_sections,
      fig_width = fig_width,
      fig_height = fig_height,
      theme = theme,
      highlight = highlight,
      ...
    ))
  
  ## swap template afterwards in order to retain original mathjax functionality
  ## and to include widget-enabling header scripts in document body
  template <- system.file("resources", "fragment.html", package="BiocStyle")
  idx <- which(config$pandoc$args=="--template") + 1L
  config$pandoc$args[idx] <- template
  
  ## one needs to run the post processor processing headers before bookdown, but
  ## process captions only after bookdown
  
  post = config$post_processor
  
  config$post_processor <- function(metadata, input, output, clean, verbose) {
    if (is.function(post)) output = post(metadata, input, output, clean, verbose)
    
    lines <- readUTF8(output)
    
    ## replace footnotes with sidenotes
    lines = process_footnotes(lines)
    
    ## set captions as sidenotes
    lines = process_captions(lines)
    
    ## add navigation
    if (isTRUE(navlinks))
      lines <- add_navigation(lines, navlinks_top, navlinks_level)
    
    writeUTF8(lines, output)
    output
  }
  
  config
}

add_refs_header = function(lines) {
  refs <- grep('^<div id="refs" class="references">$', lines)
  
  if ( !isTRUE(grepl('<h1>.*</h1>', lines[refs-1L])) )
    lines <- c(lines[1:refs],
               '<h1>References</h1>',
               lines[(refs+1L):length(lines)])
  
  lines
}

process_headers = function(lines) {
  template <- '<h%s>%s</h%s>'
  pattern <- sprintf(template, '([1-5])', '(.*)', '\\1')
  
  headers <- grep(pattern, lines, value=TRUE)
  levels <- as.integer(gsub(pattern, '\\1', headers))
  
  matches <- regexpr(pattern, lines)
  
  f <- function(header, level) {
    sub(pattern, sprintf(template, level, '\\2', level), header)
  }
  
  regmatches(lines, matches) <- mapply(f, headers, levels+1L, USE.NAMES = FALSE)
  
  lines
}

add_navigation = function(lines, top, level) {
  ## match to all section div's
  pattern <- sprintf('^<div id="(.*)" class="section level([1-%d]).*">$', level)
  idx <- grep(pattern, lines)
  sections <- lines[idx]
  sections_length <- length(sections)
  
  ## extract section metadata
  section_ids <- sub(pattern, '#\\1', sections)
  section_levels <- as.integer(sub(pattern, '\\2', sections))
  section_names <- sub('<h([1-6])>(?:<span class="header-section-number">[0-9.]*</span> )?(.*)</h\\1>', '\\2', lines[idx+1L])
  
  seq_indices = seq_along(sections)
  
  ## add virtual top section at level 0
  if (isTRUE(top)) {
    section_ids <- c("#top", section_ids)
    section_levels <- c(0L, section_levels)
    section_names <- c("Top", section_names)
    seq_indices <- seq_indices + 1L
  }
  
  ## index of previous section on the same level
  section_prev <- sapply(seq_indices, function(i) {
    level = section_levels[i]
    neighbor <- which(section_levels==level)
    level_up <- which(section_levels==level-1L)
    level_up <- max(level_up[level_up<i], 0L)
    neighbor <- neighbor[neighbor<i & neighbor>level_up]
    neighbor[length(neighbor)][1L]
  })
  
  ## index of next section on the same level
  section_next <- sapply(seq_indices, function(i) {
    level = section_levels[i]
    neighbor <- which(section_levels==level)
    level_up <- which(section_levels==level-1L)
    level_up <- min(level_up[level_up>i], sections_length+1L)
    neighbor[neighbor>i & neighbor<level_up][1L]
  })
    
  ## index of parent section
  section_up <- sapply(seq_indices, function(i) {
    level = section_levels[i]
    level_up <- which(section_levels[1:i]==level-1L)
    level_up[length(level_up)][1L]
  })
  
  ## join navigation for sections and their immediate subsections
  merged <- c(FALSE, idx[-length(idx)]+2L == idx[-1L])
  merged_ids <- which(merged) + isTRUE(top)
  section_ids[merged_ids] <- section_ids[merged_ids - 1L]
  idx <- idx[!merged]
  
  ## move links to next subsections from fused sections up to their parents
  section_down <- rep(NA_integer_, sections_length)
  section_down[c(merged[-1L], FALSE)] <- section_next[merged]
  
  ## create links
  create_link <- function(v=0L, id=section_ids[v], name=section_names[v], icon)
    ifelse(is.na(v), "", sprintf('<span class="nav-icon">%s</span> <a href="%s">%s</a><br/>', icon, id, name))
  
  links <- paste0(
    create_link(id="#TOC", name="Table of Contents", icon="&#8803;"),
    create_link(section_next, icon="&#9656;"),
    create_link(section_prev, icon="&#9666;"),
    create_link(section_up, icon="&#9652;"),
    create_link(section_down, icon="&#9662;")
  )[!merged]
  
  ## preallocate the results vector and populate it with original lines
  idx_length <- length(idx)
  res <- vector(mode = "character", length = length(lines)+idx_length)
  idx <- idx + seq_len(idx_length)
  res[-idx] <- lines
  
  ## insert links
  res[idx] <- sprintf('<p class="sidenote">%s</p>', links)
  
  res
}

process_captions = function(lines) {
  ## convert table captions enlosed in <caption> tags into <p class="caption">
  pattern <- '(^<caption>)(.*)(</caption>$)'
  idx <- grep('^<table', lines)
  idx <- idx[grepl(pattern, lines[idx+1L])]
  captions <- lines[idx+1L]
  lines[idx+1L] <- lines[idx]
  lines[idx] <- gsub(pattern, '<p class="caption">\\2</p>', captions)
  
  ## append 'sidenote' CSS class to figure and table captions
  lines = gsub('(?<=^<p class="caption)(?=">)', ' sidenote', lines, perl=TRUE)
  
  lines
}

#' @importFrom knitr current_input hook_pdfcrop hook_plot_md hook_plot_tex knit_hooks knit_theme opts_chunk opts_hooks opts_knit set_header
#' @importFrom rmarkdown metadata knitr_options output_format pandoc_available pandoc_version
#' @importFrom stats setNames
#' @importFrom utils packageVersion
#' @importFrom yaml yaml.load
NULL

modifyLines <- function (lines, from, to, replace = TRUE, before = FALSE, 
                         offset = 0L, fixed = TRUE, insert = "") {
  from <- grep(from, lines, fixed=fixed)[1L]
  
  # exit if nothing found
  if (is.na(from)) return(lines)
  
  if (missing(to)) {
    to <- from
  } else {
    to <- grep(to, lines, fixed=fixed)
    if (length(to)==0) return(lines) else to = to[to > from][1L]
  }
  
  offset <- rep_len(offset, 2L)
  from <- from + offset[1L]
  to <- to + offset[2L]
  
  if (isTRUE(replace)) { # substitute lines from-to
    c(lines[1:(from-1L)], insert, lines[(to+1L):length(lines)])
  } else {
    if (isTRUE(before)) # insert before
      c(lines[1:(from-1L)], insert, lines[from:length(lines)])
    else # insert after
      c(lines[1:to], insert, lines[(to+1L):length(lines)])
  }
}

readUTF8 = function(file, ...) readLines(file, encoding = 'UTF-8', warn = FALSE, ...)

writeUTF8 = function(text, ...) writeLines(enc2utf8(text), ..., useBytes = TRUE)

biocTempfile = function(name) {
  biocDir = file.path(tempdir(), "BiocStyle")
  if (!dir.exists(biocDir))
    dir.create(biocDir)
  
  file.path(biocDir, name)
}

sub_ext = function(file, ext) sub("([.][[:alnum:]]+)?$", ext, file)

## recursively merge two lists
merge_lists <- function (base_list, overlay_list) {
  if (length(base_list) == 0L)
    overlay_list
  else if (length(overlay_list) == 0L)
    base_list
  else {
    merged_list <- base_list
    for (name in names(overlay_list)) {
      base <- base_list[[name]]
      overlay <- overlay_list[[name]]
      if (is.list(base) && is.list(overlay))
        merged_list[[name]] <- merge_lists(base, overlay)
      else 
        merged_list[[name]] <- overlay_list[[name]]
    }
    merged_list
  }
}

## print file content
.print.file <- function(file, scoped = FALSE) {
  type = unlist(strsplit(file, split=".", fixed=TRUE))
  type = tolower(type[length(type)])
  
  paste(c(
    switch(type, 
           js  = '<script type="text/javascript">',
           css = if (isTRUE(scoped)) '<style type="text/css" scoped>'
           else '<style type="text/css">',
           NULL),
    ## actual file content
    readLines(file),
    switch(type, 
           js  = '</script>\n',
           css = '</style>\n',
           NULL)
  ), collapse = '\n')
}

loadBioconductorStyleFile <- function(file, opts=NULL) {
  sprintf("\\RequirePackage[%s]{%s}",
          paste(opts, collapse = ","),
          sub(".sty$", "", file))
}

copyResource <- function(file, dir) {
  filename <- basename(file)
  file.copy(file, file.path(dir, filename))
  setNames(filename, names(file))
}

test_maketitle <- function()
{
    dir <- system.file("unitTests", "cases", package="BiocStyle")
    fls <- dir(dir, pattern="maketitle_test.*.Rnw", full=TRUE)

    fls <- normalizePath(fls)
    pwd <- setwd(tempdir())
    on.exit(setwd(pwd))

    for (fl in fls) {
        checkTrue(file.copy(fl, rnw <- file.path(tempdir(), basename(fl))))
        tex <- utils::Sweave(rnw)
        pdf <- tools::texi2pdf(tex)
    }
}

#' Use Bioconductor CSS style to format HTML vignettes
#' 
#' This function sets the Bioconductor style sheet to provide a consistent
#' style across Bioconductor HTML vignettes.
#' 
#' Use is described in the \sQuote{Bioconductor CSS Style} vignette.
#' 
#' @param css.files character vector containing the location of additional
#' \code{.css} files.
#' @param self.contained logical(1), should the content of the CSS
#' \code{stylesheet} files be included into the html file or should they be
#' saved as separate files.
#' @param links.target logical(1), should external links open in new browser
#' tab/window.
#' @return No value is returned. The function is called for its side effect of
#' setting the \code{markdown} and/or \code{knitr} specific options controlling
#' the inclusion of the Bioconductor CSS style file in the HTML output.
#' @author Andrzej Oleś <andrzej.oles@@embl.de>, 2014-2015
#' @keywords manip
#' @examples
#' 
#' ## location of the .css file
#' BiocStyle:::bioconductor.css
#' 
#' @export
markdown <-
    function(css.files, self.contained = TRUE, links.target = TRUE)
{
    ## load the package to expose macros
    require(BiocStyle, quietly = TRUE)
    
    ## set document title class as in R Markdown v2
    cat( .print.file(file.path(resources, "html", "setDocumentTitleClass.js")) )

    ## set up target of external links
    if ( isTRUE(links.target) )
        cat( .print.file(file.path(resources, "html", "setExternalLinksTarget.js")) )
    
    ## set up CSS
    
    ## get the version of markdown renderer

    rmarkdown.version <-
        if ( is.null(opts_knit$get("rmarkdown.version")) ) {
            1L
        } else 2L
    
    ## in case of rmarkdown v1 insert directly into document body to
    ## circumvent issues with preview in R Studio, unless called from
    ## 'knit2html' or the vignette builder
    calls <- sapply(sys.calls(), function(x) as.character(x)[1])
    pattern <- c("knit2html", "buildVignettes")
    
    insert.into.body <-
        rmarkdown.version == 1L && all(sapply(pattern, function(x) {
            length(grep(x, calls, fixed = TRUE))
        }) == 0)
    
    if (insert.into.body)
        cat(.print.file(bioconductor.css, scoped = TRUE))
    else
        options(markdown.HTML.stylesheet = bioconductor.css)
    
    if ( !missing(css.files) ) {
        ## fail save
        css.files = css.files[file.exists(css.files)]

        if ( length(css.files) > 0 ) {
            if (insert.into.body)
                cat(.print.file(css.files, scoped = TRUE))      
            else
                options(markdown.HTML.header = if(isTRUE(self.contained)) {
                    ## insert the contents of the CSS files into the
                    ## HTML document
                    sapply(css.files, .print.file)
                } else {
                    ## insert relative links to the .css files
                    txt <- '<link rel="stylesheet" type="text/css" href="./%s"/>'
                    sprintf(txt, css.files)
                })
        }
    }
    
    invisible()
}


## macro definitions

#' Link to packages on Bioconductor, CRAN and GitHub
#' 
#' Functions for adding links to Bioconductor, CRAN and GitHub packages into R
#' Markdown documents.
#' 
#' Use \code{Biocpkg} for Bioconductor software, annotation,
#' experiment data, and workflow packages. When \code{vignette=NULL},
#' the function automatically includes a link to the package landing
#' page, the version of which depends on the current Bioconductor
#' version (i.e. if run in a devel environment, it will point towards
#' the devel landing page; otherwise it will point to the release
#' landing page).
#'
#' Use \code{Biocbook} for Bioconductor webbooks. This will create a
#' link to the index page of the book rather than to the landing page
#' of the package used for the book's deployment.
#'
#' Use \code{CRANpkg} for R packages available on CRAN. The function
#' automatically includes a link to the master CRAN landing page.
#' 
#' Use \code{Githubpkg} for R packages available on GitHub. The \code{repo}
#' should include the repository address in the format username/repo[/subdir].
#' If \code{package} is missing, the package name is assumed to be equal the
#' repository name and is extracted from \code{repo}.
#' 
#' For R packages which are not available on Bioconductor, CRAN or GitHub, use
#' \code{Rpackage}.
#' 
#' @param pkg character(1), package name
#' @param vignette character(1), basename of vignette link, including
#'     html or pdf extension, e.g., "work-0-intro.html".
#' @param label character(1) label used to identify the package or
#'     vignette. If \code{NULL}, defaults to \code{pkg}.
#' @param repo Repository address in the format username/repo[/subdir]
#' @return Markdown-formatted character vector containing a
#'     hyperlinked package name. If \code{vignette != NULL}, the
#'     address of the specified vignette is returned.
#'
#' @author Andrzej Oleś <andrzej.oles@@embl.de>, 2014-2015
#' @examples
#' 
#' 
#' ## link to a Bioconductor software package
#' Biocpkg("IRanges")
#' 
#' ## link to a Bioconductor annotation package
#' Biocpkg("org.Mm.eg.db")
#'
#' ## link to a Bioconductor experiment data package
#' Biocpkg("affydata")
#'
#' ## link to a Bioconductor workflow
#' Biocpkg("simpleSingleCell")
#' Biocpkg(
#'     "simpleSingleCell",
#'     vignette = "work-0-intro.html",
#'     label = "Episode 1: analyzing scRNA-seq data with R/Bioconductor"
#' )
#'
#' ## link to a CRAN package
#' CRANpkg("data.table")
#' 
#' ## link to an R package on GitHub
#' Githubpkg("rstudio/rmarkdown")
#'
#' @name macros
NULL

#' @rdname macros
#' @importFrom BiocManager version
#' @export
Biocpkg <- function(pkg, vignette = NULL, label = NULL) {
    url <- file.path("https://bioconductor.org/packages", version(), pkg)
    if (!is.null(vignette)) {
        url <- file.path(url, "vignettes", vignette)
    }
    labelled_link(pkg, label, url)
}

labelled_link <- function(pkg, label, url) {
    if (is.null(label)) {
        Rpackage(sprintf("[%s](%s)", pkg, url))
    } else {
        sprintf("[%s](%s)", label, url)
    }
}

#' @rdname macros
#' @export
Biocbook <- function(pkg, label = NULL) {
    url <- file.path("http://bioconductor.org/books", version(), pkg)
    labelled_link(pkg, label, url)
}

#' @rdname macros
#' @export
Biocannopkg <- function(pkg) {
    Biocpkg(pkg)
}

#' @rdname macros
#' @export
Biocexptpkg <- function(pkg) {
    Biocpkg(pkg)
}

#' @rdname macros
#' @export
CRANpkg <- function(pkg) {
    cran <- "https://CRAN.R-project.org/package"
    fmt <- '[%s](%s=%s)'
    Rpackage( sprintf(fmt, pkg, cran, pkg) )
}

#' @rdname macros
#' @export
Rpackage <- function(pkg) {
    sprintf('*%s*', pkg)
}

#' @rdname macros
#' @export
Githubpkg <- function(repo, pkg) {
    github <- "https://github.com"
    ## extract package name
    if (missing(pkg)) {
      names = strsplit(repo, split = "/", fixed = TRUE)[[1L]]
      pkg = names[length(names)]
    }
    Rpackage( sprintf('[%s](%s/%s)', pkg, github, repo) )
}

## yaml header convenience functions

#' Specify Rmarkdown document metadata
#' 
#' Helper functions for including metadata in the document header.
#' 
#' Use \code{doc_date} to include document compilation date in the document
#' metadata field 'date', and \code{pkg_ver} for package version specification
#' in the 'package' field.
#' 
#' @param pkg character(1), package name
#' @return Markdown-formatted character string.
#' @author Andrzej Oleś <andrzej.oles@@embl.de>, 2014-2015
#' @keywords manip
#' @examples
#' 
#' 
#' ## current date
#' doc_date()
#' 
#' ## package name with version
#' pkg_ver("BiocStyle")
#' 
#' @name helpers
NULL

#' @rdname helpers
#' @export
pkg_ver <- function(pkg) {
  pkgVer <- if (pkg=="packageName") "X.Y.Z"  else packageVersion(pkg)
  paste(pkg, pkgVer)
}

#' @rdname helpers
#' @export
doc_date <- function() {
  sub("^0", "", format(Sys.Date(), '%d %B %Y'))
}


#' Output format of an R Markdown document
#' 
#' Helper function to determine the document's current pandoc output format.
#' 
#' The function is useful for defining different behavior depending on the
#' output format, e.g. figure settings.
#' 
#' @return A character string specifying the pandoc output format.
#' @author Andrzej Oleś <andrzej.oles@@embl.de>, 2016
#' @examples
#' 
#' \dontrun{
#' ## Switch between SVG and PDF figures depending on document output format
#' knitr::opts_chunk$set(
#'   dev = switch(output(), html = "svg", latex = "pdf")
#'   )
#' }
#' 
#' @export output
output = function () {
  opts_knit$get("rmarkdown.pandoc.to")
}

# a list of knitr options as returned by rmarkdown::knitr_options
.knitr_options = function() {
  list(
    opts_knit = list(
      width = .width()
    ),
    opts_chunk = .opts_chunk,
    knit_hooks = list(
      # remove figure margins
      crop = hook_pdfcrop
    ),
    opts_hooks = .opts_hooks,
    opts_template = NULL
  )
}

.opts_chunk = list(
  error = FALSE,
  fig.scap = NA, # disable default short caption extraction
  crop = TRUE
)

.opts_hooks = list(
  # options fig.small and fig.wide have precedance over fig.env
  fig.small = function(options) {
    if (isTRUE(options$fig.small)) {
      options$fig.env = "smallfigure"
    }
    options
  },
  fig.wide = function(options) {
    if (isTRUE(options$fig.wide)) {
      options$fig.env = "figure*"
    }
    options
  },
  # Set default plot dimensions if user provided no values
  # Important: the hooks are processed in the order they are defined here
  # regardless of the order of options in the code chunk header
  fig.width = function(options) {
    if (is.na(options$fig.width)) {
      options$fig.width = switch(options$fig.env, 7,# fallback to knitr default
                                 "smallfigure" = 5,
                                 "figure*" = 10,
                                 "figure" = 8)
      # override 'fig.height' if 'fig.asp' is set (as plain knitr does)
      if (is.numeric(options$fig.asp))
        options$fig.height = options$fig.width * options$fig.asp
    }
    
    if(is.null(options$out.width) && grepl(x = options$fig.env, pattern = "*figure*"))
      options$out.width = "100%" 
    
    options
  },
  fig.height = function(options) {
    if ( is.na(options$fig.height) ){
      options$fig.height = 5
    }
    
    options
  }
)

test_default_figure_options <- function() {
    filename <- rmarkdown::draft(tempfile(fileext = ".Rmd"), edit = FALSE,
                                 template="html_document", package = "BiocStyle")
    output_file <- tempfile(fileext = ".html")
    rmarkdown::render(filename, output_file = output_file, quiet = TRUE)

    ## with default options, we expect all three figures to have out.width = "100%"
    html <- BiocStyle:::readUTF8(output_file)
    checkTrue(length( grep(pattern = 'width="100%"', x = html) ) == 3L)
    file.remove(output_file)
    
    ## modify the small figure to have a define out.width and check it's respected
    lines <- BiocStyle:::readUTF8(filename)
    lines <- gsub(x = lines, pattern = "fig.small=TRUE", 
                  replacement = 'fig.small=TRUE, out.width="500px"')
    BiocStyle:::writeUTF8(lines, filename)
    rmarkdown::render(filename, output_file = output_file, quiet = TRUE)
    html <- BiocStyle:::readUTF8(output_file)
    checkTrue(length( grep(pattern = 'width="100%"', x = html) ) == 2L)
    checkTrue(length( grep(pattern = 'width="500px"', x = html) ) == 1L)
    file.remove(output_file)
    
    file.remove(filename)
}

#' Use Bioconductor style to format R Markdown HTML output
#' 
#' Format for converting from R Markdown to an Bioconductor HTML document.
#' 
#' \code{BiocStyle::html_document} format extends the
#' \code{\link[rmarkdown]{html_document}} format. See the
#' \href{http://rmarkdown.rstudio.com/html_document_format.html}{online
#' documentation} and the package vignette "Authoring R Markdown Vignettes" for
#' additional details on using the format,
#' 
#' @param toc logical(1), \code{TRUE} to include a table of contents in the
#' output
#' @param number_sections logical(1), \code{TRUE} to number section headings
#' @param fig_width numeric(1), default width (in inches) for figures
#' @param fig_height numeric(1), default width (in inches) for figures
#' @param self_contained numeric(1), \code{TRUE} to produce a standalone HTML
#' file with no external dependencies, using data: URIs to incorporate the
#' contents of linked scripts, stylesheets, images, and videos. Note that even
#' for self contained documents MathJax is still loaded externally (this is
#' necessary because of it's size).
#' @param css character, one or more css files to include
#' @param pandoc_args character, additional command line options to pass to
#' pandoc
#' @param \dots Additional arguments passed to
#' \code{\link[rmarkdown]{html_document}}
#' @param titlecaps logical(1), \code{TRUE} to use the emphasize the first
#' sentence in figure and table captions as title
#' @return R Markdown output format to pass to \code{\link[rmarkdown]{render}}
#' @author Andrzej Oleś <andrzej.oles@@embl.de>, 2014-2017
#' @seealso \code{\link[BiocStyle]{pdf_document}},
#' \code{\link[BiocStyle]{md_document}}
#' @examples
#' 
#' \dontrun{
#' 
#' # simple invocation
#' render("input.Rmd", BiocStyle::html_document())
#' 
#' # specify options
#' render("input.Rmd", BiocStyle::html_document(toc = FALSE))
#' }
#' 
#' @export html_document
html_document <- function(toc = TRUE,
                          number_sections = TRUE,
                          fig_width = NA,
                          fig_height = NA,
                          self_contained = TRUE,
                          css = NULL,
                          pandoc_args = NULL,
                          ...,
                          ## BiocStyle specific arguments:
                          titlecaps = TRUE) {
  
  ## load the package to expose macros
  require(BiocStyle, quietly = TRUE)
  
  ## customize the default rmarkdown template
  template <- create_html_template()
  
  # append any user-provided CSS files
  css <- c(bioconductor.css, css)
  
  post_processor = function(metadata, input, output, clean, verbose) {
    x = readUTF8(output)
    
    ## insert author affiliations
    x <- modifyLines(x, from='<!-- AUTH AFFIL -->', insert=auth_affil_html(metadata))
    
    ## format caption titles
    if (isTRUE(titlecaps))
      x = caption_titles(x)
    
    ## replace footnotes with sidenotes
    x = process_footnotes(x)
    
    ## wrap tables in a container
    x = process_tables(x)
    
    ## remove css that hardcodes <pre> tag styling
    x = modify_css(x)
    
    ## js that adds accessibility tools to TOC
    x = add_toc_nav_js(x)
    
    writeUTF8(x, output)
    output
  }
  
  # knitr options
  knitr = merge_lists(.knitr_options(), list(
    knit_hooks = list(
      plot = function(x, options = list()) {
        out.extra = switch(options$fig.env, NULL,
                           "smallfigure" = 'class="smallfigure"',
                           "figure*" = 'class="widefigure"',
                           "figure" = NULL)
        options$out.extra = paste(options$out.extra, out.extra)
        hook_plot_md(x, options)
      }
    )
  ))
  
  # mask section numbering in bookdown in order to get a global rather than
  # per-section numbering of figures/tables
  
  number_sections_override <- number_sections

  base_format <- function(..., number_sections) {
    rmarkdown::html_document(..., number_sections = number_sections_override)
  }
  
  # call the rmarkdown::html_document function with `template="default"` and
  # substitute the template only afterwards in order to retain some of the
  # original functionality such as mathjax, floating toc, and code folding.
  config <- output_format(
    knitr = knitr,
    pandoc = NULL,
    clean_supporting = self_contained,
    pre_processor = pre_processor,
    post_processor = post_processor,
    base_format = bookdown::html_document2(
      base_format = base_format,
      toc = toc,
      number_sections = FALSE,
      fig_width = fig_width,
      fig_height = fig_height,
      self_contained = self_contained,
      css = css,
      ...)
  )
  
  ## override some default pandoc args; we use this low-level approach rather 
  ## than passing them in 'pandoc_args' to 'rmarkdown::html_document' because 
  ## rmarkdown just concatenates base and overlay argument lists which does not 
  ## allow for substitution
  pandoc_args = c(pandoc_args, c("--template", template))
  
  arg_names <- c("--email-obfuscation", "--template")
  arg_names <- arg_names[arg_names %in% pandoc_args]
  
  idx = match(arg_names, pandoc_args)
  
  ## substitute arguments
  config$pandoc$args [
    match(arg_names, config$pandoc$args) + 1L 
    ] <- pandoc_args[idx + 1L]
  
  ## append the rest
  config$pandoc$args <- c(config$pandoc$args, pandoc_args[-c(idx, idx+1L)])
  
  config
}

# modify the default rmarkdown template
create_html_template <- function() {
  lines <- readUTF8(system.file("rmd", "h", "default.html", package = "rmarkdown"))
  
  template <- biocTempfile("template.html")
  
  ## placeholder for author affiliation block which is inserted during postprocessing
  lines <- modifyLines(lines, from='$for(author)$', to='$endfor$', insert='<!-- AUTH AFFIL -->')
  
  lines <- modifyLines(lines, from='<div class="abstract">', to='</div>', insert=c(
    '<h4 class="abstract">Abstract</h4>',
    '$abstract$',
    '$endif$',
    '$if(package)$',
    '<h4 class="package">Package</h4>',
    '<p>$package$</p>'))
  
  lines <- modifyLines(lines, from='<div id="$idprefix$TOC">', replace=FALSE, before=TRUE, insert="<h1>Contents</h1>")
  
  ## modify some inline CSS
  lines <- modifyLines(lines, from='^\\.toc-content \\{', to = '\\}', fixed=FALSE)
  
  lines <- modifyLines(lines, from='^\\code \\{', to = '\\}', fixed=FALSE)
  
  for (i in 1:2) 
    lines <- modifyLines(lines, from='^  pre:not\\(\\[class\\]\\) \\{', to = '\\}', fixed=FALSE)
  
  lines <- modifyLines(lines=lines, from='^\\.main-container \\{', to = '\\}', fixed=FALSE, offset=c(1L, -1L), insert=c(
    '  max-width: 828px;',
    '  margin-left: auto;',
    '  margin-right: auto;'))
  
  lines <- modifyLines(lines, from='^div\\.tocify \\{', to = '\\}', fixed=FALSE, offset=c(1L, -1L), insert=c(
    '  width: 20%;',
    '  max-width: 246px;',
    '  max-height: 85%;'))
  
  ## use the modified code folding script
  lines <- modifyLines(lines=lines, from='<script src="$navigationjs$/codefolding.js"></script>', insert=
                         sprintf('<script src="%s"></script>', file.path(resources, "html", "codefolding.js")))
  
  ## Automatic equation numbering
  lines <- modifyLines(lines=lines, from='$if(mathjax-url)$', replace=FALSE, before=TRUE, insert=c(
    '<script type="text/x-mathjax-config">',
    '  MathJax.Hub.Config({',
    # '    TeX: {',
    # '      TagSide: "right",',
    # '      equationNumbers: {',
    # '        autoNumber: "AMS"', 
    # '      }',
    # '    },',
    '    "HTML-CSS": {',
    '      styles: {',
    '        ".MathJax_Display": {',
    '           "text-align": "center",',
    '           padding: "0px 150px 0px 65px",',
    '           margin: "0px 0px 0.5em"',
    '        },',
    '        "@media screen and (max-width: 991px)": {',
    '            ".MathJax_Display": {',
    '               "text-align": "center",',
    '               padding: "0 0 0 0"',
    '            }',
    '         }',
    '      }',
    '    }',
    '  });',
    '</script>'))
  
  lines <- modifyLines(lines = lines, from = "</head>", replace = FALSE, before = TRUE,
                       insert = c(
                           '<script>',
                           'function toggle_visibility(id1) {',
                           '  var e = document.getElementById(id1);',
                           '  e.style.display = ((e.style.display!="none") ? "none" : "block");',
                           '}',
                           '</script>',
                           ''))

  writeUTF8(lines, template)
  
  template
}

caption_titles = function(lines) {
  regex_template = '%s(\\(#%s:[-/[:alnum:]]+\\))[[:space:]]*(.*?)([[:space:]]*((\\.[[:space:]])|(\\.$)|($)))'
  replacement = '\\1<span class="caption-title">\\2</span><br>'
  
  # tables: match to <caption>...</caption> lines
  regex = sprintf(regex_template, '(?<=^<caption>)[[:space:]]*', 'tab')
  lines = gsub(regex, replacement, lines, perl=TRUE)
  
  # figures: match to figure labels preceded by '<p class="caption'
  regex = sprintf(regex_template, '', 'fig')
  idx =  which(grepl(regex, lines))
  idx = idx[vapply(idx, function(i) any(grepl('^<p class="caption', lines[i-0:1])), logical(1L))]
  lines[idx] = gsub(regex, replacement, lines[idx])
  
  lines
}

process_footnotes = function(lines) {
  fn_label = paste0(knitr::opts_knit$get("rmarkdown.pandoc.id_prefix"), "fn")
  
  ## match to footnotes block
  i = grep(pattern = '^<div class="footnotes[^"]*"[^>]*>', x = lines)
  if (length(i) == 0L)
    return(lines)
  j = which(lines == '</div>')
  j = min(j[j > i])
  
  ## extract footnotes and their ids
  regex = sprintf('<li id="%s([0-9]+)"><p>(.+)<a href="#%sref\\1"([^>]*)>.{1,2}</a></p></li>', fn_label, fn_label)
  footnote_lines = paste(lines[i:j], collapse = '\n')
  fns = unlist(regmatches(footnote_lines, gregexpr(regex, footnote_lines)))
  ids = as.integer(gsub(regex, '\\1', fns))
  fns = gsub(regex, '\\2', fns)
  
  # remove footnotes at the bottom
  lines = lines[-(i:j)]
  
  # replace footnotes with sidenotes
  cls = if (pandoc_available('2.0')) 'footnote-ref' else 'footnoteRef'
  for (i in seq_along(ids)) {
    id = ids[i]
    lines = sub(
      sprintf(
        '<a href="#%s%d" class="%s" id="%sref%d"><sup>%d</sup></a>',
        fn_label, id, cls, fn_label, id, id),
      sprintf(paste0(
        '<label for="sidenote-%d" class="sidenote-number">%d</label>',
        '<label for="sidenote-%d" class="margin-toggle" onclick="toggle_visibility(\'sidenote-%d\')">%d</label>',
        '<span class="sidenote" id="sidenote-%d"><span class="sidenote-number">%d</span> %s</span>'
        ), id, id, id, id, id, id, id, fns[i]),
      lines, fixed=TRUE)
  }
  lines
}

## We wrap tables in a <div> so they can fill the screen on mobile
## but also scroll if they overflow horizontally
process_tables = function(lines) {
  lines = gsub("<table", "<div class='horizontal-scroll'><table", lines, fixed = TRUE)
  lines = gsub("</table>", "</table></div>", lines, fixed = TRUE)
  lines
}

modify_css = function(lines) {
  ## Since rmarkdown v 2.7.0 the CSS below gets injected during HTML creation, 
  ## rather than being taken from the template.html. We remove it again here.
  lines = gsub("pre:not\\(\\[class\\]\\) \\{ background-color: white \\}", "", lines, fixed = FALSE)
  
  ## "pre code {" appears multiple times, so modifyLines() isn't appropriate here
  lines_collapsed = paste0(lines, collapse = "\n")
  lines_collapsed = gsub("pre code {\n  padding: 0;\n}\n", replacement = "", x = lines_collapsed, fixed = TRUE)
  lines = strsplit(lines_collapsed, split = "\n", fixed = TRUE)[[1]]
  
  lines
}

add_toc_nav_js = function(lines) {
  lines = modifyLines(lines = lines, from = "</body>", replace = FALSE, before = TRUE,
                      insert = .print.file( file.path(resources, "html", "addTocAccessibility.js")))
  lines
}

## ----macro, echo=FALSE--------------------------------------------------------
macro <- function(name, pkg, description)
    sprintf('`%s("%s")` %s %s', name, pkg,
            description, do.call(name, list(pkg)))
inline <- function(cmd) sprintf('`` `r %s` ``', cmd)

## ----no-cap, fig.alt="A default sized example figure without a caption."------
plot(cars)

## ----plot, fig.cap="Regular figure. The first sentence of the figure caption is automatically emphasized to serve as figure title.", echo=FALSE----
plot(cars)

## ----wide, fig.cap="Wide figure. A plot produced by a code chunk with option `fig.wide = TRUE`.", fig.wide=TRUE, echo=FALSE----
plot(cars)

## ----small, fig.cap="Small figure. A plot produced by a code chunk with option `fig.small = TRUE`.", fig.small=TRUE, echo=FALSE----
plot(cars)

## ----mtcars-------------------------------------------------------------------
knitr::kable(
  head(mtcars[, 1:8], 10), caption = 'A table of the first 10 rows of `mtcars`.'
)

## ----sessionInfo, echo=FALSE--------------------------------------------------
sessionInfo()

#' @importFrom rmarkdown draft
.vignette_template <-
    function(destination, mode = c("html", "pdf"))
{
    stopifnot(
        is.character(destination), length(destination) == 1L, !is.na(destination),
        !file.exists(destination), file.exists(dirname(destination))
    )
    mode <- match.arg(mode)

    doc <- paste(mode, "document", sep = "_")
    rmarkdown::draft(
        destination, template = doc, package = "BiocStyle", edit = FALSE
    )
    destination
}

#' @title Create 'Rmd' vignette templates for HTML or PDF output
#'
#' @rdname use_vignette
#' 
#' @param destination character(1) file path to destination. The 
#'     directory of the destination must already exist, and the file 
#'     must not. The default creates a file in the temporary 
#'     directory, and so is removed when the R session ends.
#' 
#' @examples
#' use_vignette_html()
#'
#' @export
use_vignette_html <- 
    function(destination = tempfile(fileext = ".Rmd")) 
{
    .vignette_template(destination, "html")
}

#' @rdname use_vignette
#'
#' @export
use_vignette_pdf <- 
    function(destination = tempfile(fileext = ".Rmd")) 
{
    .vignette_template(destination, "pdf")
}

test_pdf_document <- function() {
  filename <- rmarkdown::draft(tempfile(fileext = ".Rmd"), edit = FALSE,
                               template="pdf_document", package = "BiocStyle")
  lines <- BiocStyle:::readUTF8(filename)
  lines <- BiocStyle:::modifyLines(lines, from = "title:", replace = FALSE, 
                       insert='subtitle: "Vignette Subtitle"')
  BiocStyle:::writeUTF8(lines, filename)
  rmarkdown::render(filename)
  unlink(filename)
}

#' Defunct functions in package \sQuote{BiocStyle}
#'
#' These functions are defunct and no longer available.
#'
#' The following functions are no longer available; use the replacement
#' indicated below:
#' \itemize{
#'   \item \code{latex_old, latex2: \link{latex}}
#'   \item \code{pdf_document_old, pdf_document2: \link{pdf_document}}
#'   \item \code{html_document_old, html_document2: \link{html_document}}
#' }
#'
#' @name BiocStyle-defunct
NULL

#' Convert to a BiocStyle markdown document
#' 
#' Enable BiocStyle macros and author affiliations in markdown documents.
#' 
#' The \code{BiocStyle::md_document} format extends the base
#' \code{\link[rmarkdown]{md_document}} format by means of specifing complex
#' author affiliations. It also loads the BiocStyle package namespace to enable
#' the use of helper functions, such as \code{\link{Biocpkg}}.
#' 
#' @param toc \code{TRUE} to include a table of contents in the output
#' @param ... Additional function arguments to pass to the base R Markdown
#' \code{\link[rmarkdown]{md_document}} output formatter
#' @return R Markdown output format to pass to \code{\link[rmarkdown]{render}}
#' @author Andrzej Oleś <andrzej.oles@@embl.de>, 2015-2016
#' @seealso \code{\link[BiocStyle]{html_document}},
#' \code{\link[BiocStyle]{pdf_document}}
#' @examples
#' 
#' \dontrun{
#' 
#' rmarkdown::render("input.Rmd", BiocStyle::md_document())
#' }
#' 
#' @export md_document
md_document <- function(toc = TRUE, ...) {
  
  ## load the package to expose macros
  require(BiocStyle, quietly = TRUE)
  
  base_format = rmarkdown::md_document(toc = toc, md_extensions = "-markdown_in_html_blocks", ...)
  
  ## append kramdown toc label at the very beginning of document body
  if ( isTRUE(toc) ) {
    base_format$pre_processor <- function(metadata, input_file, runtime, knit_meta, files_dir, output_dir) {
      lines <- readUTF8(input_file)
      
      doc <- rmarkdown:::partition_yaml_front_matter(lines)
      lines <- c(doc$front_matter, "{:toc}", doc$body)
      
      writeUTF8(lines, input_file)
    }
  }
  
  post_processor <- function(metadata, input_file, output_file, clean, verbose) {
    lines <- readUTF8(output_file)
    
    # convert absolute paths to relative, see https://github.com/rstudio/rmarkdown/issues/587
    lines = gsub(file.path(dirname(output_file), ''), '', lines, fixed=TRUE)
    
    # move all headers one level down (for proper formatting when embedded in the website)
    pattern <- '^(#+ )'
    lines <- gsub(pattern, '#\\1', lines)
    
    # make sure there is a newline before headers (pandoc removes them if preceeded by, e.g., <p>...</p>)
    idx <- grep(pattern, lines)
    idx <- idx[lines[idx - 1L] != ""]
    if ((l = length(idx))) {
      idx <- idx + 0:(l-1L)
      res <- vector(mode="character", length(lines)+l)
      res[-idx] <- lines
      lines <- res
    }
    
    # remove empty line between TOC and kramdown reference name
    lines <- modifyLines(lines, from="{:toc}", offset=-1L, insert=NULL)
    
    header <- NULL
    
    # add document title
    if ( !is.null(metadata$title) )
      header <- c(sprintf("# %s", metadata$title), "{:.no_toc}", "", header)
    
    # add author affiliations
    if ( !is.null((auth_affil <- auth_affil_html(metadata))) )
      header <- c(header, auth_affil, "")
    
    # TOC header
    header <- c(header, "## Contents", "{:.no_toc}", "")
    
    lines <- c(header, lines)
    
    writeUTF8(lines, output_file)
    output_file
  }
  
  knitr = knitr_options(opts_knit = list(width = 80L))
  
  # return format
  output_format(knitr = knitr,
                pandoc = list(args = "--atx-headers"),
                clean_supporting = FALSE,
                post_processor = post_processor,
                base_format = base_format)
}

## r markdown pre-processor function modifying document front matter
pre_processor = function(metadata, input_file, runtime, knit_meta, files_dir, output_dir) {
  entries = c()
  
  ## the metadata may contain r code, so we extract it from the knitted 
  ## document instead of using the 'metadata' argument
  lines <- readUTF8(input_file)
  lines <- rmarkdown:::partition_yaml_front_matter(lines)
  front_matter <- lines$front_matter
  body <- lines$body
  
  ## process the package field
  idx = grep("^package:", front_matter)
  if ( length(idx)==1L ) {
    ## use yaml parser to clean-up input from possible quotes etc.
    yaml_list <- yaml.load(front_matter[idx])
    package <- trimws(yaml_list$package)
    ## append version number if the field contains valid package name 
    if ( isTRUE(grepl("^[[:alnum:].]+[[:alnum:]]+$", package)) )
      front_matter[idx] <- sprintf("package: %s", pkg_ver(package))
  }
  
  ## add date if missing
  if ( is.null(metadata$date) )
    entries <- c(entries, sprintf("date: %s", doc_date()))
  
  if( is.null(metadata$`link-citations`) )
    entries <- c(entries, "link-citations: true")
  
  ## append new metadata entries to yaml front matter
  l = length(front_matter)
  front_matter <- c(front_matter[-l],
                    entries,
                    front_matter[l])
  
  ## identify a references section
  refs_header_loc <- grepl(pattern = "^# References", x = body, fixed = FALSE)
  if(any(refs_header_loc)) {
    ## only insert an Appendix header if there's anything after the references
    idx <- which(refs_header_loc)
    refs_is_last_element <- idx == length(body) || all(nzchar(body[(idx+1):length(body)]) == FALSE) 
    if(!refs_is_last_element) {
      body <- modifyLines(lines = body, from = "^# References", 
                          replace = FALSE, fixed = FALSE,
                          insert = '<div id="refs"></div>\n\n# (APPENDIX) Appendix {-}\n')
    }
  }
  
  writeUTF8(c(front_matter, body), input_file)
  
  NULL
}

---
title: "Authoring R Markdown vignettes"
author:
- name: Andrzej Oleś
  affiliation: European Molecular Biology Laboratory, Heidelberg, Germany
- name: Wolfgang Huber
  affiliation: European Molecular Biology Laboratory, Heidelberg, Germany
- name: Martin Morgan
  affiliation: Roswell Park Cancer Institute, Buffalo, NY
output:
  BiocStyle::html_document:
    toc_float: true
  BiocStyle::pdf_document: default
package: BiocStyle
bibliography: Bioc.bib
abstract: |
  Instructions on enabling _Bioconductor_ style in R Markdown vignettes.
vignette: |
  %\VignetteIndexEntry{Authoring R Markdown vignettes}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

# Prerequisites

_Bioconductor_ R Markdown format is build on top of _R_ package 
`r CRANpkg("bookdown")` [@Xie2018], which in turn relies on `r CRANpkg("rmarkdown")` and
[pandoc](http://johnmacfarlane.net/pandoc/) to compile the final output
document. Therefore, unless you are using RStudio, you will need a recent
version of [pandoc](http://johnmacfarlane.net/pandoc/) (>= 1.17.2). See the
[pandoc installation
instructions](https://github.com/rstudio/rmarkdown/blob/master/PANDOC.md) for
details on installing pandoc for your platform.


# Getting started

To enable the _Bioconductor_ style in your R Markdown vignette you need to:

- Edit the `DESCRIPTION` file by adding

        VignetteBuilder: knitr
        Suggests: BiocStyle, knitr, rmarkdown

- Specify `BiocStyle::html_document` or `BiocStyle::pdf_document` as output
format and add vignette metadata in the document header:

        ---
        title: "Vignette Title"
        author: "Vignette Author"
        package: PackageName
        output: 
          BiocStyle::html_document
        vignette: >
          %\VignetteIndexEntry{Vignette Title}
          %\VignetteEngine{knitr::rmarkdown}
          %\VignetteEncoding{UTF-8}  
        ---

The `vignette` section is required in order to instruct _R_ how to build the
vignette.^[`\VignetteIndexEntry` should match the `title` of your vignette] The
`package` field which should contain the package name is used to print the
package version in the output document header. It is not necessary to specify
`date` as by default the document compilation date will be automatically
included. See the following section for details on specifying author
affiliations and abstract.

BiocStyle's `html_document` and `pdf_document` format functions extend the
corresponding original _rmarkdown_ formats, so they accept the same arguments as
`html_document` and `pdf_document`, respectively. For example, use `toc_float:
true` to obtain a floating TOC as in this vignette.

## Use with R markdown v1

Apart from the default markdown engine implemented in the
`r CRANpkg('rmarkdown')` package, it is also possible to compile _Bioconductor_
documents with the older markdown v1 engine from the package
`r CRANpkg('markdown')`. There are some differences in setup and the resulting
output between these two engines.

To use the `r CRANpkg('markdown')` vignette builder engine:

- Edit the `DESCRIPTION` file to include

        VignetteBuilder: knitr
        Suggests: BiocStyle, knitr
        
- Specify the vignette engine in the `.Rmd` files (inside HTML comments)

        <!--
        %% \VignetteEngine{knitr::knitr}
        -->

- Add the following code chunk at the beginning of your `.Rmd` vignettes

        ```{r style, echo = FALSE, results = 'asis'}`r ''`
        BiocStyle::markdown()
        ```

The way of attaching CSS files when using `r CRANpkg('markdown')` differs from
how this is done with `r CRANpkg('rmarkdown')`. In the former case additional
style sheets can be used by providing them to the `BiocStyle::markdown`
function. To include `custom.css` file use

    ```{r style, echo = FALSE, results = 'asis'}`r ''`
    BiocStyle::markdown(css.files = c('custom.css'))
    ```


# Document header

## Author affiliations

The `author` field allows for specifying author names along with affiliation and
email information.

In the basic case, when no additional information apart from author names is
provided, these can be entered as a single character string

    author: "Single Author"
    
or a list

    author:
    - First Author
    - Second Author
    - Last Author
    
which will print as "First Author, Second Author and Last Author".

Author affiliations and emails can be entered in named sublists of the author
list. Multiple affiliations per author can be specified this way.

    author:
    - name: First Author
      affiliation: 
      - Shared affiliation
      - Additional affiliation
    - name: Second Author
      affiliation: Shared affiliation
      email: corresponding@author.com

A list of unique affiliations will be displayed below the authors, similar as in
this document.

For clarity, compactness, and to avoid errors, repeated nodes in YAML header can
be initially denoted by an anchor entered with an ampersand &, and later
referenced with an asterisk *. For example, the above affiliation metadata is
equivalent to the shorthand notation

    author:
    - name: First Author
      affiliation: 
      - &id Shared affiliation
      - Additional affiliation
    - name: Second Author
      affiliation: *id
      email: corresponding@author.com

## Abstract and running headers

Abstract can be entered in the corresponding field of the document front matter,
as in the example below.

    ---
    title: "Full title for title page"
    shorttitle: "Short title for headers"
    author: "Vignette Author"
    package: PackageName
    abstract: >
      Document summary
    output: 
      BiocStyle::pdf_document
    ---

The `shorttitle` option specifies the title used in running headers instead of
the document title.^[only relevant to PDF output]


# Style macros

```{r macro, echo=FALSE}
macro <- function(name, pkg, description)
    sprintf('`%s("%s")` %s %s', name, pkg,
            description, do.call(name, list(pkg)))
inline <- function(cmd) sprintf('`` `r %s` ``', cmd)
```

`r Biocpkg("BiocStyle")` introduces the following macros useful when referring
to _R_ packages:

* `r macro("Biocpkg", "IRanges", "for _Bioconductor_ software, annotation and
experiment data packages, including a link to the release landing page or if the
package is only in devel, to the devel landing page,")`.

* `r macro("CRANpkg", "data.table", "for _R_ packages available on CRAN,
including a link to the FHCRC CRAN mirror landing page,")`.

* `r macro("Githubpkg", "rstudio/rmarkdown", "for _R_ packages available on
GitHub, including a link to the package repository,")`.

* `r macro("Rpackage", "MyPkg", "for _R_ packages that are _not_ available on
_Bioconductor_, CRAN or GitHub;")`.

These are meant to be called inline, e.g., `r inline('Biocpkg("IRanges")')`.


# Code chunks

The line length of output code chunks is set to the optimal width of typically
80 characters, so it is not neccessary to adjust it manually through
`options("width")`.


# Figures

## Simple figures

_BiocStyle_ comes with three predefined figure sizes. Regular figures not
otherwise specified appear indented with respect to the paragraph text, as in
the example below.

```{r no-cap, fig.alt="A default sized example figure without a caption."}
plot(cars)
```

## Figure captions

Figures which have no captions, like the one above, are just placed wherever they were generated in
the _R_ code. If you assign a caption to a figure via the code chunk option
`fig.cap`, the plot will be automatically labeled and numbered^[for PDF output
it will be placed in a floating figure environment], and it will be also
possible to reference it. These features are provided by `r CRANpkg("bookdown")`,
which defines a format-independent syntax for specifying
cross-references, see Section \@ref(cross-references). The figure label is
generated from the code chunk label^[for cross-references to work the chunk
label may only contain alphanumeric characters (a-z, A-Z, 0-9), slashes (/), or
dashes (-)] by prefixing it with `fig:`, e.g., the label of a figure originating
from the chunk `foo` will be `fig:foo`. To reference a figure, use the syntax
`\@ref(label)`^[do not forget the leading backslash!], where `label` is the
figure label, e.g., `fig:foo`. For example, the following code chunk was used to
produce Figure \@ref(fig:plot).

    ```{r plot, fig.cap = "Regular figure. The first sentence...", echo = FALSE}`r ''`
    plot(cars)
    ```

```{r plot, fig.cap="Regular figure. The first sentence of the figure caption is automatically emphasized to serve as figure title.", echo=FALSE}
plot(cars)
```

## Alternative figure sizes

In addition to regular figures, _BiocStyle_ provides small and wide figures
which can be specified by `fig.small` and `fig.wide` code chunk options. Wide
figures are left-aligned with the paragraph and extend on the right margin, as
Figure \@ref(fig:wide). Small figures are meant for possibly rectangular plots
which are centered with respect to the text column, see Figure \@ref(fig:small).

```{r wide, fig.cap="Wide figure. A plot produced by a code chunk with option `fig.wide = TRUE`.", fig.wide=TRUE, echo=FALSE}
plot(cars)
```

```{r small, fig.cap="Small figure. A plot produced by a code chunk with option `fig.small = TRUE`.", fig.small=TRUE, echo=FALSE}
plot(cars)
```

## Accessibility considerations

Alt (alternative) text is used by screen readers to provide a description of an
image to visually impaired readers.  If you provide a figure caption with
`fig.cap` the same text will automatically be used as the alt text for the
image.  However, if you want to include an image without a caption, or you wish
to provide different information in the alt text to the caption, this can be
done via the `fig.alt` code chunk option.  As an example, the code chunk below
was used to include the first image shown in this section, so that its intention
could be conveyed to someone using a screen reader.

    ```{r no-cap, fig.alt="A default sized example figure without a caption."}`r ''`
    plot(cars)
    ```
    
You may also want to consider the color palette used in your figures, to ensure
they are accessible to color impaired readers.  _BiocStyle_ doesn't provide a
specific color palette, but many options are available in packages from CRAN.
There have also been [significant
efforts](https://developer.r-project.org/Blog/public/2019/11/21/a-new-palette-for-r/)
made to improve the default colors provided by `palette()` in R-4.0.0.
Bioconductor contains some domain specific packages that focus on color blind
friendly visualizations e.g. `r Biocpkg("dittoSeq")` for single-cell and bulk
RNA-sequencing data.

# Tables

Like figures, tables with captions will also be numbered and can be referenced.
The caption is entered as a paragraph starting with `Table:`^[or just `:`],
which may appear either before or after the table. When adding labels, make sure
that the label appears at the beginning of the table caption in the form
`(\#tab:label)`, and use `\@ref(tab:label)` to refer to it. For example, Table
\@ref(tab:table) has been produced with the following code.

```markdown
Fruit   | Price
------- | -----
bananas | 1.2
apples  | 1.0
oranges | 2.5
  
: (\#tab:table) A simple table. With caption.
```
        
Fruit   | Price
------- | -----
bananas | 1.2
apples  | 1.0
oranges | 2.5
  
: (\#tab:table) A simple table. With caption.
    
The function `knitr::kable()` will automatically generate a label for a table
environment, which is the chunk label prefixed by `tab:`, see Table
\@ref(tab:mtcars).

```{r mtcars}
knitr::kable(
  head(mtcars[, 1:8], 10), caption = 'A table of the first 10 rows of `mtcars`.'
)
```


# Equations

To number and reference equations, put them in equation environments and append
labels to them following the syntax `(\#eq:label)`^[due to technical constraints
equation labels must start with `eq:`], e.g.,

```latex
\begin{equation}
  f\left(k\right) = \binom{n}{k} p^k\left(1-p\right)^{n-k}
  (\#eq:binom)
\end{equation}
```
renders the equation below.

\begin{equation}
  f\left(k\right) = \binom{n}{k} p^k\left(1-p\right)^{n-k}
  (\#eq:binom)
\end{equation} 

You may then refer to Equation \@ref(eq:binom) by `\@ref(eq:binom)`. Note that
in HTML output only labeled equations will appear numbered.


# Cross-references

Apart from referencing figures (Section \@ref(figures)), tables (Section
\@ref(tables)), and equations (Section \@ref(equations)), you can also use the
same syntax `\@ref(label)` to reference sections, where `label` is the section
ID. By default, Pandoc will generate IDs for all section headers, e.g., `# Hello
World` will have an ID `hello-world`. In order to avoid forgetting to update the
reference label after you change the section header, you may also manually
assign an ID to a section header by appending `{#id}` to it.

When a referenced label cannot be found, you will see two question marks like
\@ref(non-existing-label), as well as a warning message in the _R_ console when
rendering the document.


# Margin notes

Footnotes are displayed as side notes on the right margin^[this is a side note
entered as a footnote], which has the advantage that they appear close to the
place where they are defined.

# Bibliography

If you wish to include a list of references you can use the special section title
`# References`^[it must be exactly this!], and provide a bibtex file in the vignette header.
A list of all references used in the text will be automatically
inserted after this heading. By default the references section will continue the 
section level numbering used throughout the document.  To suppress the numbering,
as seen in this vignette, you can use the Bookdown syntax for 
[unnumbered sections](https://bookdown.org/yihui/rmarkdown-cookbook/unnumbered-sections.html) 
e.g. `{-}` or `{.unnumbered}`.

Everything after the list of references will be considered as 
appendices. See below for an example of the formatting change.  
For more details on how to include a bibliography please see the relevant chapter of
[bookdown.org](https://bookdown.org/yihui/rmarkdown-cookbook/bibliography.html#bibliography).


# References {.unnumbered}

# Session info

Here is the output of `sessionInfo()` on the system on which this document was
compiled running pandoc `r rmarkdown::pandoc_version()`:

```{r sessionInfo, echo=FALSE}
sessionInfo()
```

---
title: "Vignette Title"
author:
- name: First Author
  affiliation: First Author's Affiliation
- name: Second Author
  affiliation: Second Author's Affiliation
  email: corresponding@author.com
package: packageName
output:
  BiocStyle::pdf_document
abstract: |
  Description of your vignette
vignette: |
  %\VignetteIndexEntry{Vignette Title}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

# Getting started

To enable the _Bioconductor_ style in your R Markdown vignette you need to include the following in the `DESCRIPTION` file:

    VignetteBuilder: knitr
    Suggests: BiocStyle, knitr, rmarkdown


# Style macros

_BiocStyle_ introduces the following macros for referring to _R_ packages:

* `r Biocpkg("IRanges")`, for _Bioconductor_ software, annotation and experiment data packages,
* `r CRANpkg("data.table")`, for _R_ packages available on CRAN,
* `r Githubpkg("rstudio/rmarkdown")`, for _R_ packages available on GitHub,
* `r Rpackage("MyPkg")`, for _R_ packages that are _not_ available on _Bioconductor_, CRAN or GitHub.


# Figures

Assign captions to figures in the code chunk option `fig.cap` to automatically number them, and to be able to reference them, see Figure \@ref(fig:plot). The figure label is generated from the code chunk label by prefixing it with `fig:`.

```{r plot, fig.cap="Regular figure. The first sentence of the figure caption is automatically emphasized to serve as figure title.", echo=FALSE}
plot(cars)
```

Small and wide figures can be specified by `fig.small` and `fig.wide` code chunk options.

```{r small, fig.cap="Small figure. A plot produced by a code chunk with option `fig.small = TRUE`.", fig.small=TRUE, echo=FALSE}
plot(cars)
```

```{r wide, fig.cap="Wide figure. A plot produced by a code chunk with option `fig.wide = TRUE`.", fig.wide=TRUE, echo=FALSE}
plot(cars)
```


# Equations

To number and reference equations, put them in equation environments and assign labels to them, see Equation \@ref(eq:binom).

\begin{equation}
  f\left(k\right) = \binom{n}{k} p^k\left(1-p\right)^{n-k}
  (\#eq:binom)
\end{equation}


# Tables

Like figures, tables with captions will also be numbered and can be referenced, see Table \@ref(tab:table).

Fruit   | Price
------- | -----
bananas | 1.2
apples  | 1.0
oranges | 2.5

: (\#tab:table) A simple table. With caption.


# Cross-references

Apart from referencing figures (Section \@ref(figures)), tables (Section \@ref(tables)), and equations (Section \@ref(equations)), you can also use the same syntax to refer to sections by their default labels generated by pandoc.


# Side notes

Footnotes are displayed as side notes on the right margin^[this is a side note entered as a footnote], which has the advantage that they appear close to the place where they are defined.

# References {.unnumbered}


# Session info

```{r sessionInfo, echo=FALSE}
sessionInfo()
```

%% \maketitle & title header tests -- title, author, maketitle in body

\documentclass{article}

<<style, echo=FALSE, results=tex>>=
BiocStyle::latex()
@

\begin{document}

\title{An Amazing Document}
\author{author1\footnote{a@abc.com}}
\maketitle

\clearpage
hello world

\end{document}

%\VignetteIndexEntry{Bioconductor LaTeX Style 2.0}
%\VignettePackage{BiocStyle}
%\VignetteEngine{utils::Sweave}

\documentclass{article}

<<style, eval=TRUE, echo=FALSE, results=tex>>=
BiocStyle::latex()
@ 

\usepackage{booktabs} % book-quality tables

\newcommand\BiocStyle{\Rpackage{BiocStyle}}
\newcommand\knitr{\Rpackage{knitr}}
\newcommand\rmarkdown{\Rpackage{rmarkdown}}
\newcommand\Sweave{\software{Sweave}}
\newcommand\latex[1]{{\ttfamily #1}}

\makeatletter
\def\thefpsfigure{\fps@figure}
\makeatother

\newcommand{\exitem}[3]{%
  \item \latex{\textbackslash#1\{#2\}} #3 \csname#1\endcsname{#2}.%
}

\title{\Bioconductor\ \LaTeX\ Style 2.0}
\author[1]{Andrzej Ole\'s}
\author[2]{Martin Morgan}
\author[1]{Wolfgang \mbox{Huber}}
\affil[1]{European Molecular Biology Laboratory, Heidelberg, Germany}
\affil[2]{Roswell Park Cancer Institute, Buffalo, NY}

\begin{document}

\maketitle

\begin{abstract}
This vignette describes the new Bioconductor \LaTeX\ document style. It aims
to be at once a demonstration of the features of the style and a guide to their use.
The vignette focuses on the use with documents typeset as markup .Rnw files and 
processed using \Sweave\cite{Leisch2002} or \knitr\cite{Xie2014}.
The same style is also applied to \R\ markdown .Rmd files which are rendered to PDF output format. 
%For markdown-specific syntax see the \emph{Bioconductor \R\ markdown style} package vignette.
\end{abstract}

\packageVersion{\Sexpr{BiocStyle::pkg_ver("BiocStyle")}}

Report issues on \url{https://github.com/Bioconductor/BiocStyle/issues}

\newpage

\tableofcontents

\newpage

\section{Authoring package vignettes}

\subsection{Getting started}

\BiocStyle\ provides standard formatting of documents rendered
to PDF output format. It consists of a \LaTeX\ document style definition which
can be used with either plain \Sweave, \knitr, or \rmarkdown. 
To enable \BiocStyle\ in your package vignette follow the instructions below 
depending on the input format.

\subsubsection{\Sweave}

To use with \Sweave, add the following to your package \file{DESCRIPTION} file:
\begin{verbatim}
    Suggests: BiocStyle
\end{verbatim}
and add this code chunk to the preamble\footnote{before the {\ttfamily \textbackslash begin\{document\}} command, and preferably right after the {\ttfamily \textbackslash documentclass} definition} of your .Rnw file:
\begin{verbatim}
    <<style-Sweave, eval=TRUE, echo=FALSE, results=tex>>=
    BiocStyle::latex()
    @ 
\end{verbatim}

See \Rcode{?latex} for additional options. It is not necessary for Sweave 
vignettes in general to include {\ttfamily\textbackslash usepackage\-\{Sweave\}}
in the document preamble, and it will give you an error if it appears before 
the call to the style function.\footnote{because \BiocStyle\ loads 
{\ttfamily Sweave.sty} in order to override some defaults}
Other \LaTeX\ packages attached by \BiocStyle\ are listed in Appendix~\ref{sec:pkgs}.
Avoid loading these packages yourself, as doing so may cause unexpected side
effects and often leads to document compilation errors.

\subsubsection{\knitr{}}\label{sec:knitr} 

To use with \CRANpkg{knitr}, add the following to the \file{DESCRIPTION} file:
\begin{verbatim}
    VignetteBuilder: knitr
    Suggests: BiocStyle, knitr
\end{verbatim}
this to the top of the .Rnw file:
\begin{verbatim}
    %\VignetteEngine{knitr::knitr}
\end{verbatim}
and this to the preamble:
\begin{verbatim}
    <<style-knitr, eval=TRUE, echo=FALSE, results="asis">>=
    BiocStyle::latex()
    @ 
\end{verbatim}

See \Rcode{?latex} for additional options, and Appendix~\ref{sec:pkgs} for a list of automatically attached \LaTeX\ packages.

\subsubsection{R markdown}

To enable \BiocStyle\ in your \R\ markdown vignette you need to:

\begin{enumerate}
  
  \item Edit to the \file{DESCRIPTION} file by adding

  \begin{verbatim}
    VignetteBuilder: knitr
    Suggests: BiocStyle, knitr, rmarkdown
  \end{verbatim}
    
  \item Specify \Rcode{BiocStyle::pdf\_document} as the output format and add vignette metadata to the document header:

  \begin{verbatim}
    ---
    title: "Vignette Title"
    author: "Vignette Author"
    output: 
      BiocStyle::pdf_document
    vignette: >
      %\VignetteIndexEntry{Vignette Title}
      %\VignetteEngine{knitr::rmarkdown}
      %\VignetteEncoding{UTF-8}  
    ---
  \end{verbatim}
  
\end{enumerate}
  
The \verb+vignette+ section is required in order to instruct \R\ how to build the vignette. Note that \verb+\VignetteIndexEntry+ should match the \verb+title+ of your vignette.
  
See the \emph{\href{http://www.bioconductor.org/packages/devel/bioc/vignettes/BiocStyle/inst/doc/AuthoringRmdVignettes.html}{Authoring R Markdown vignettes}} package vignette for
more details on authoring markdown documents with \BiocStyle.


\subsection{Workflow}

Provided that \BiocStyle\ has been installed, a convenient way to build the
vignette as it is being written is with the command
\begin{verbatim}
    R CMD Sweave --pdf vignette.Rnw
\end{verbatim}
A short-cut useful for checking the \LaTeX{} portion of vignettes is
to toggle evaluation of code chunks to \Rcode{FALSE}.
\begin{verbatim}
    SWEAVE_OPTIONS="eval=FALSE" R CMD Sweave --pdf vignette.Rnw
\end{verbatim}

When using \knitr, the command to process the vignette is
\begin{verbatim}
    R CMD Sweave --engine=knitr::knitr --pdf vignette.Rnw
\end{verbatim}
For \R\ markdown vignettes, set the engine to \verb+knitr::rmarkdown+.
\begin{verbatim}
    R CMD Sweave --engine=knitr::rmarkdown --pdf vignette.Rmd
\end{verbatim}

By default, \knitr\ automatically caches results of vignette
chunks, greatly accelerating the turnaround time required for
edits. Both the default and \knitr\ incantations create PDF
files using \verb+texi2dvi --pdf+; many versions of this software
incorrectly display non-breaking spaces as a tilde, \verb|~|. This can be
remedied (as on the \Bioconductor{} build system) with a final run of
\begin{verbatim}
    R CMD texi2dvi --pdf vignette.tex
    R CMD pdflatex vignette.tex
\end{verbatim}

\section{Structuring the document}

\subsection{Title page}

\warning[Important]{most of the methods described here only work once \BiocStyle\ is loaded;
therefore, specify title and authors \emph{after} the code chunk calling \Rfunction{BiocStyle::latex()}.}

\subsubsection{Title and running headers}

Create a title and running headers by defining \verb+\bioctitle+
and \verb+\author+ in the preamble
\begin{verbatim}
    \bioctitle[Short title for headers]{Full title for title page}
    %% also: \bioctitle{Title used for both header and title page} 
    %% or... \title{Title used for both header and title page} 
    \author{Vignette Author\thanks{\email{user@domain.com}}}
\end{verbatim}
and use \verb+\maketitle+ at the beginning of the document to output the title and author information.

\subsubsection{Authors and affiliations}

As illustrated above, use the \verb+\email+ command to enter hyperlinked email addresses
typeset in monospace font. Multiple author affiliations can be specified with the help of
the \LaTeX\ package \latex{authblk} which is automatically loaded by \BiocStyle.
See the following examples for typical use cases. 

If your vignette has just a single author, use standard \LaTeX\ syntax
to enter the author and affiliation information separated by a new line.
You can provide the email address in \verb+\thanks+, or in another new line 
after the affiliation.

\begin{verbatim}
    \author{Vignette Author\thanks{\email{user@domain.com}}\\
            Author's Affiliation}
\end{verbatim}

In case of multiple authors, you can specify the author and affiliation information in different manners depending on 
the number of affiliations. In the basic case, when you don't provide authors'
affiliation, or all the authors share the same affiliation, you can just use the regular \verb+\author+ command:%
\footnote{the usual author separator {\ttfamily \textbackslash and} won't work because the affiliations are typeset in footnote mode}

\begin{verbatim}
    \author{
      First Author\thanks{\email{first@author.com}} and
      Second Author\thanks{\email{second@author.com}}\\
      Shared Affiliation
    }
\end{verbatim}

To provide different affiliations, some of which are potentially shared, 
enter each author separately and use \verb+\affil+
to specify affiliation. The authors will appear all in one (possibly continued) line
with automatic footnotes, and the affiliations are displayed in separate lines below.\footnote{as on this vignette's title page}

\begin{verbatim}
    \author{First Author}
    \author{Second Author}
    \affil{First and Second authors' shared affiliation}
    \author{Third Author\thanks{\email{corresponding@author.com}}}
    \affil{Third author's affiliation}
\end{verbatim}

In the case when some authors have more than one affiliation, or the authors with 
a shared affiliation do not come one after another, you need to manually define
footnote markers as optional arguments to \verb+\author+ and \verb+\affil+ commands, as in the following example.

\begin{verbatim}
    \author[1,2]{First Author\thanks{\email{user@domain.com}}}
    \author[1]{Second Author}
    \author[2]{Third Author}
    \affil[1]{First and Second authors' affiliation}
    \affil[2]{First and Third authors' affiliation}
\end{verbatim}


\subsubsection{Abstract, package version, and table of contents}

Abstract can be entered using the standard \verb+abstract+ environment:

\begin{verbatim}
    \begin{abstract}
      Short summary of the vignette's content.
    \end{abstract}
\end{verbatim}

It is recommended to include information on the specific 
package version described in the vignette. The following line\footnote{substitute 
\latex{packageName} by the name of your package} inserts
this information automatically and in a properly formatted manner.

\verb|    \packageVersion{\|\verb|Sexpr{BiocStyle::pkg_ver("packageName")}}|

Use \verb+\tableofcontents+ to include a hyperlinked table of contents (TOC), and
\verb+\section+, \verb+\subsection+, \verb+\subsubsection+ for
structuring your document.  It is a good practice to start the first section
on a new page by calling \verb+\newpage+ after the TOC:

\begin{verbatim}
    \tableofcontents
    \newpage
    \section{My First Section}
\end{verbatim}

\subsection{Style macros}

\BiocStyle\ introduces the following additional markup styling commands
useful in typical \Bioconductor{} vignettes.\\\\
%%
Software:
\begin{itemize}
  \item \verb+\R{}+ and \verb+\Bioconductor{}+ to reference \R{} software and
  the \Bioconductor{} project.
  \exitem{software}{GATK}{to reference third-party software, e.g.,}
\end{itemize}
%\vspace{1em}
%%
Packages:
\begin{itemize}
  \exitem{Biocpkg}{IRanges}{for \Bioconductor{} software, annotation and experiment data packages, including a
  link to the release landing page or if the package is only in devel, to the devel landing page.}
  \exitem{CRANpkg}{data.table}{for \R{} packages available on CRAN, including
  a link to the FHCRC CRAN mirror landing page,}
  \exitem{Githubpkg}{rstudio/rmarkdown}{for \R{} packages available on GitHub, including a link to the package repository,}
  \exitem{Rpackage}{MyPkg}{for \R{} packages that are \emph{not} available on
  \Bioconductor{} or CRAN,}
\end{itemize}
%\vspace{1em}
%%
Code:
\begin{itemize}
  \exitem{Rfunction}{findOverlaps}{for functions}
  \exitem{Robject}{olaps}{for variables}
  \exitem{Rclass}{GRanges}{when referring to a formal class}
  \exitem{Rcode}{log(x)}{for \R{} code,}
\end{itemize}
%\vspace{1em}
%%
Communication:
\begin{itemize}
  \exitem{bioccomment}{additional information for the user}{communicates}
  \exitem{warning}{common pitfalls}{signals}
  \exitem{fixme}{incomplete functionality}{provides an indication of}
\end{itemize}
For all of the above message types, the default opening word can be overriden in an optional argument, e.g.~\latex{\textbackslash{fixme}[TODO]\{missing functionality\}} produces \fixme[TODO]{missing functionality}.

General:
\begin{itemize}
  \exitem{email}{user@domain.com}{to provide a linked email address,}
  \exitem{file}{script.R}{for file names and file paths}
\end{itemize}

\subsection{Code chunks}

%\BiocStyle\ changes the default setting of the \verb+error+ option

\BiocStyle\ uses some heuristics to automatically adjust the line length\footnote{\Rcode{options("width")}} 
of output code chunks such that it matches the document font size setting.\footnote{for the default font size of 10pt the code width is 80 columns}
If for some reason you wish to alter it, use the \Rcode{width} argument in the call to \Rfunction{BiocStyle::latex}.

%---------------------------------------------------------
\subsection{Figures}\label{sec:figures}
%---------------------------------------------------------

<<fig, echo=FALSE, fig=TRUE, include=FALSE, width=8, height=5>>=
par(mar=c(4,4,0.5,0.5))
plot(cars)
<<widefig, echo=FALSE, fig=TRUE, include=FALSE, width=10, height=5>>=
par(mar=c(4,4,0.5,0.5))
plot(cars)
<<smallfig, echo=FALSE, fig=TRUE, include=FALSE, width=5, height=5>>=
par(mar=c(4,4,0.5,0.5))
plot(cars)
<<squarefig, echo=FALSE, fig=TRUE, include=FALSE, width=10, height=10>>=
par(mar=c(4,4,0.5,0.5))
plot(cars)
@

In addition to the standard \verb+\figure+ environment \BiocStyle\ provides special
environments for small and wide figures. These three environments differ in size, 
default aspect ratio\footnote{device dimensions are automatically set only by \knitr\ and \rmarkdown, and not by \Sweave}, 
and horizontal placement on the the page. See the following
table for details, and Figures~\ref{fig:figure}, \ref{fig:wide} and \ref{fig:small} 
for examples.

\begin{table}
  \begin{small}
  \begin{tabular}{llp{6.5mm}lll}
    \toprule
    environment & description & \multicolumn{2}{l}{output width} & device dimensions & aspect ratio\\
    \midrule
    \latex{figure} & regular figure & 120mm &($\approx$4.72in) & 8 x 5 in & 1.6\\
    \latex{figure*} & wide figure & 150mm  &($\approx$5.90in)& 10 x 5 in & 2.0\\
    \latex{smallfigure} & small figure & 75mm &($\approx$2.95in)& 5 x 5 in & 1.0 \\
    \bottomrule
  \end{tabular}
  \end{small}
  \label{tab:figs}
\end{table}

\begin{figure}
  \includegraphics{\jobname-fig}
  \caption{\label{fig:figure}Regular figure. A plot displayed with the \latex{\textbackslash figure} environment.}
\end{figure}

\begin{figure*}
  \includegraphics{\jobname-widefig}
  \caption{\label{fig:wide}Wide figure. A plot displayed
  with the {\ttfamily\textbackslash figure*} environment.}
\end{figure*}

\begin{smallfigure}
  \includegraphics{\jobname-smallfig}
  \caption{\label{fig:small}Small figure. A plot displayed with the \latex{\textbackslash{}smallfigure} environment.}
\end{smallfigure}

The \verb+figure+ environment produces regular figures which are left-indented
and right-aligned with the paragraph, as Figure~\ref{fig:figure}.
To utilize the whole available width use the \verb+figure*+ environment.
It produces figures which are left-aligned with the paragraph
and extend on the right margin (Figure~\ref{fig:wide}).
The \verb+smallfigure+ environment is meant for possibly rectangular plots
which are about half as wide as the paragraph (Figure~\ref{fig:small}).
The default placement specifier for \BiocStyle\ floats is \latex{\thefpsfigure}, 
which typically outputs them in the place where they are defined. 
The first sentence of figure captions is emphasized to serve as figure title. 
This feature can be disabled be setting the argument \Rcode{titlecaps=FALSE} in the call to \Rfunction{BiocStyle::latex}.

To use figure environments in \Sweave, write explicit \LaTeX\ code which inserts
them in combination with the \Sweave\ option \verb+include=FALSE+. 
For example, Figure~\ref{fig:wide} was produced with the following code.

\begin{verbatim}
    <<widefig, echo=FALSE, fig=TRUE, include=FALSE, width=10, height=5>>=
    par(mar=c(4,4,0.5,0.5))
    plot(cars)
    @
\end{verbatim}
\begin{verbatim}
    \begin{figure*}
      \includegraphics{\jobname-widefig}
      \caption{\label{fig:wide}Wide figure. A plot displayed
      with the {\ttfamily\textbackslash figure*} environment.}
    \end{figure*}
\end{verbatim}

In \knitr\ and \rmarkdown\ the output environment for a graphics-producing code
chunk can be specified in \Rcode{fig.env} chunk option, e.g.~set 
\Rcode{fig.env='smallfigure'} to get \verb+\begin{smallfigure}+. 
It is also possible to specify the wide and small figure
environments by setting \Rcode{fig.wide} or \Rcode{fig.small} option to \Rcode{TRUE}.
The following two \knitr\ code chunks are equivalent and produce the same output, similar to Figure~\ref{fig:small}.
\begin{verbatim}
    <<smallfig, fig.cap="Small figure.", fig.env="smallfigure">>=
    plot(cars)
    @
\end{verbatim}
\begin{verbatim}
    <<smallfig, fig.cap="Small figure.", fig.small=TRUE>>=
    plot(cars)
    @
\end{verbatim}

Specify \Rcode{fig.width} and/or \Rcode{fig.height} to override the default 
device dimensions listed in the table on page~\pageref{tab:figs}. To adjust the 
aspect ratio use \Rcode{fig.asp}. For example, the following code would produce 
a full-width square plot, as in Figure~\ref{fig:square}.

\begin{verbatim}
    <<square, fig.wide=TRUE, fig.asp=1>>=
    plot(cars)
    @
\end{verbatim}

\begin{figure*}
  \includegraphics{\jobname-squarefig}
  \caption{Full-width square figure. Use {\ttfamily fig.wide=TRUE, fig.asp=1} code chunk options to enter with \knitr\ or \rmarkdown.\label{fig:square}}
\end{figure*}

When \knitr\ is used plots are cropped so that blank margins around them are removed to make better use of the space in the output document\footnote{otherwise one often has to struggle with \Rcode{par} to set appropriate margins}. This feature can be switched off by setting the chunk option \Rcode{knitr::opts\_chunk\$set(crop = NULL)}.

\subsubsection{\latex{\textbackslash incfig} convenience macro}\label{incfig}

Besides the usual \LaTeX\ capabilities (the \verb+figure+ environment and
\verb+\includegraphics+ command) \BiocStyle\ defines a macro 

\verb+    \incfig[placement]{filename}{width}{title}{caption}+

which expects four arguments:
\begin{description}[font=\ttfamily]
\item[filename] The name of the figure file, also used as the label by
  which the float can be referred to by \verb+\ref{}+. Some
  \Sweave\ and \knitr\ options place figures in a
  subdirectory; unless \Rcode{short.fignames=TRUE} is set the full file name,
  including the subdirectory and any prefixes, should be provided. By default,
  these are \file{<sweavename>-} for \Sweave\ and \file{figure/} for
  \knitr. Also note the different naming scheme used by
  \knitr: figure files are named \file{<chunkname>-i} where \emph{i}
  is the number of the plot generated in the chunk. 
\item[width] Figure width.
\item[title] A short title, used in the list of figures and
  printed in bold as the first part of the caption.
\item[caption] Extended description of the figure.
\end{description}
The optional \textbf{\ttfamily placement} specifier controls where the figure is placed
on page; it takes the usual values allowed by \LaTeX{} floats, i.e., a list
containing \verb+t+, \verb+b+, \verb+p+, or \verb+h+, where letters enumerate
permitted placements\footnote{if no placement specifier is given, the default
\latex{\thefpsfigure} is assumed}.

For \verb+incfig+ with \Sweave\, use
\begin{verbatim}
    <<figureexample, fig=TRUE, include=FALSE, width=4.2, height=4.6>>=
    par(mar=c(4,4,0.5,0.5))
    v = seq(0, 60i, length=1000)
    plot(abs(v)*exp(v), type="l", col="Royalblue")
    @
    \incfig{\jobname-figureexample}{0.5\textwidth}{A curve.}
      {The code that creates this figure is shown in the code chunk.}
    as shown in Figure~\ref{\jobname-figureexample}.
\end{verbatim}
This results in
<<figureexample, fig=TRUE, include=FALSE, width=4, height=4>>=
par(mar=c(4,4,0.5,0.5))
v = seq(0, 60i, length=1000)
plot(abs(v)*exp(v), type="l", col="Royalblue")
@
\incfig{\jobname-figureexample}{0.5\textwidth}{A curve.}
  {The code that creates this figure is shown in the code chunk.}
as shown in Figure~\ref{\jobname-figureexample}.  

When option \Rcode{short.fignames=TRUE} is set, figure names used by
\verb+\incfig+ and \verb+\ref+ do not contain any prefix and are
identical to the corresponding code chunk labels (plus figure number in case of
\knitr). For example, in \Sweave\ the respective code for the above
example would be \verb+\incfig{figureexample}{...}{...}{...}+ and
\verb+\ref{figureexample}+.

For \verb+\incfig+ with \knitr, use the option
\Rcode{fig.show='hide'} rather than \Rcode{include=FALSE}. The
\knitr-equivalent code for
Figure~\ref{LatexStyle2-figureexample} is:
\begin{verbatim}
    <<figureexample, fig.show='hide', fig.width=4, fig.height=4>>=
    par(mar=c(4,4,0.5,0.5))
    v = seq(0, 60i, length=1000)
    plot(abs(v)*exp(v), type="l", col="Royalblue")
    @
\end{verbatim}
Note the difference in option names setting the figure width and
height compared to \Sweave.  Unless
\Rcode{short.fignames=TRUE} is set, use the default \file{figure/}
prefix when inserting and referring to figures, e.g.:
\begin{verbatim}
    \incfig{figure/figureexample-1}{0.5\textwidth}{A curve.}
      {The code that creates this figure is shown in the code chunk.}
\end{verbatim}
A custom prefix for figure file names can be passed to
\Rfunction{latex} in the \Rcode{fig.path} argument. When
\Rcode{short.fignames=TRUE}, figures can be referred to directly by
code chunk labels, i.e., \verb+\incfig{figureexample-1}...+ and
\verb+\ref{figureexample-1}+.

\subsection{Equations}

When referencing equations, e.g.~\eqref{equation}, use \verb+\eqref+ to ensure proper label formatting.
%
\begin{equation}\label{equation}
  \sin^2 \theta + \cos^2 \theta \equiv 1
\end{equation}

\subsection{Footnotes}

One of the distinctive features of the style is an asymmetric page layout 
with a wide margin on the right. This provides additional space for ancillary information in side notes. These can be entered in footnotes\footnote{this is a side note entered using the {\ttfamily \textbackslash footnote} command} typeset as margin notes, which has the advantage that the notes appear close to the place where they are defined. 

%---------------------------------------------------------
\subsection{Bibliography}
%---------------------------------------------------------

\Rfunction{BiocStyle::latex} has default argument
\Rcode{use.unsrturl=TRUE} to automatically format bibliographies using
\software{natbib}'s unsrturl style. There is no need to explicitly
include \software{natbib}, and it is an error to use a
\verb+\bibliographystyle+ command. The \software{unsrturl.bst} format,
e.g., \cite{Leisch2002, Xie2014}, supports
hyperlinks to DOI and PubMed IDs but not \verb+\citet+ or
\verb+\citep+.

To use a bibliography style different from \verb+unsrturl+, set
\Rcode{use.unsrturl=FALSE} and follow normal \LaTeX{} conventions.


\bibliography{Bioc}

\newpage

\appendix

%---------------------------------------------------------
\section{Session info}
%---------------------------------------------------------

Here is the output of \Rfunction{sessionInfo()} on the system on which
this document was compiled:
<<sessionInfo, results=tex, echo=FALSE>>=
toLatex(sessionInfo())
@


\section{Attached \LaTeX\ packages}\label{sec:pkgs}

<<latexPkgs, echo=FALSE>>=
bioconductor.sty = readLines(BiocStyle:::bioconductor.sty)
pattern = "^\\\\RequirePackage(\\[.*\\])?\\{([[:alnum:]]+)\\}.*"
pkgs = sub(pattern, "\\2", grep(pattern, bioconductor.sty, value = TRUE))
# add hyperref which is not captured by the regex
pkgs = c(pkgs, "hyperref")
# list sorted and unique names
pkgs = sort(unique(pkgs))
latexpkgs = paste0("\\\\latex{", pkgs, "}", collapse=", ")
@

\BiocStyle\ loads the following \LaTeX\ packages:

\Sexpr{latexpkgs}.


\section{Known issues}

\subsection{Compatibility with the \Rpackage{xtable} \R\ package}

\BiocStyle\ does not support tables produced by the \R\ package \CRANpkg{xtable} in plain \Sweave\ documents.
This limitation does not apply to documents compiled with \knitr. If you would like to 
continue using \Rpackage{xtable} in your \BiocStyle-enabled \Sweave\ document, please consider migrating it to \knitr. See \Rcode{?knitr::Sweave2knitr} for automatic conversion, and Sections~\ref{sec:knitr} and \ref{sec:figures} for details on using \BiocStyle\ with \knitr.


\end{document}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/use_vignette.R
\name{use_vignette_html}
\alias{use_vignette_html}
\alias{use_vignette_pdf}
\title{Create 'Rmd' vignette templates for HTML or PDF output}
\usage{
use_vignette_html(destination = tempfile(fileext = ".Rmd"))

use_vignette_pdf(destination = tempfile(fileext = ".Rmd"))
}
\arguments{
\item{destination}{character(1) file path to destination. The 
directory of the destination must already exist, and the file 
must not. The default creates a file in the temporary 
directory, and so is removed when the R session ends.}
}
\description{
Create 'Rmd' vignette templates for HTML or PDF output
}
\examples{
use_vignette_html()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/md_document.R
\name{md_document}
\alias{md_document}
\title{Convert to a BiocStyle markdown document}
\usage{
md_document(toc = TRUE, ...)
}
\arguments{
\item{toc}{\code{TRUE} to include a table of contents in the output}

\item{...}{Additional function arguments to pass to the base R Markdown
\code{\link[rmarkdown]{md_document}} output formatter}
}
\value{
R Markdown output format to pass to \code{\link[rmarkdown]{render}}
}
\description{
Enable BiocStyle macros and author affiliations in markdown documents.
}
\details{
The \code{BiocStyle::md_document} format extends the base
\code{\link[rmarkdown]{md_document}} format by means of specifing complex
author affiliations. It also loads the BiocStyle package namespace to enable
the use of helper functions, such as \code{\link{Biocpkg}}.
}
\examples{

\dontrun{

rmarkdown::render("input.Rmd", BiocStyle::md_document())
}

}
\seealso{
\code{\link[BiocStyle]{html_document}},
\code{\link[BiocStyle]{pdf_document}}
}
\author{
Andrzej Oleś <andrzej.oles@embl.de>, 2015-2016
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BiocStyle-defunct.R
\name{BiocStyle-defunct}
\alias{BiocStyle-defunct}
\title{Defunct functions in package \sQuote{BiocStyle}}
\description{
These functions are defunct and no longer available.
}
\details{
The following functions are no longer available; use the replacement
indicated below:
\itemize{
  \item \code{latex_old, latex2: \link{latex}}
  \item \code{pdf_document_old, pdf_document2: \link{pdf_document}}
  \item \code{html_document_old, html_document2: \link{html_document}}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/markdown.R
\name{macros}
\alias{macros}
\alias{Biocpkg}
\alias{Biocbook}
\alias{Biocannopkg}
\alias{Biocexptpkg}
\alias{CRANpkg}
\alias{Rpackage}
\alias{Githubpkg}
\title{Link to packages on Bioconductor, CRAN and GitHub}
\usage{
Biocpkg(pkg, vignette = NULL, label = NULL)

Biocbook(pkg, label = NULL)

Biocannopkg(pkg)

Biocexptpkg(pkg)

CRANpkg(pkg)

Rpackage(pkg)

Githubpkg(repo, pkg)
}
\arguments{
\item{pkg}{character(1), package name}

\item{vignette}{character(1), basename of vignette link, including
html or pdf extension, e.g., "work-0-intro.html".}

\item{label}{character(1) label used to identify the package or
vignette. If \code{NULL}, defaults to \code{pkg}.}

\item{repo}{Repository address in the format username/repo[/subdir]}
}
\value{
Markdown-formatted character vector containing a
    hyperlinked package name. If \code{vignette != NULL}, the
    address of the specified vignette is returned.
}
\description{
Functions for adding links to Bioconductor, CRAN and GitHub packages into R
Markdown documents.
}
\details{
Use \code{Biocpkg} for Bioconductor software, annotation,
experiment data, and workflow packages. When \code{vignette=NULL},
the function automatically includes a link to the package landing
page, the version of which depends on the current Bioconductor
version (i.e. if run in a devel environment, it will point towards
the devel landing page; otherwise it will point to the release
landing page).

Use \code{Biocbook} for Bioconductor webbooks. This will create a
link to the index page of the book rather than to the landing page
of the package used for the book's deployment.

Use \code{CRANpkg} for R packages available on CRAN. The function
automatically includes a link to the master CRAN landing page.

Use \code{Githubpkg} for R packages available on GitHub. The \code{repo}
should include the repository address in the format username/repo[/subdir].
If \code{package} is missing, the package name is assumed to be equal the
repository name and is extracted from \code{repo}.

For R packages which are not available on Bioconductor, CRAN or GitHub, use
\code{Rpackage}.
}
\examples{


## link to a Bioconductor software package
Biocpkg("IRanges")

## link to a Bioconductor annotation package
Biocpkg("org.Mm.eg.db")

## link to a Bioconductor experiment data package
Biocpkg("affydata")

## link to a Bioconductor workflow
Biocpkg("simpleSingleCell")
Biocpkg(
    "simpleSingleCell",
    vignette = "work-0-intro.html",
    label = "Episode 1: analyzing scRNA-seq data with R/Bioconductor"
)

## link to a CRAN package
CRANpkg("data.table")

## link to an R package on GitHub
Githubpkg("rstudio/rmarkdown")

}
\author{
Andrzej Oleś <andrzej.oles@embl.de>, 2014-2015
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/html_document.R
\name{html_document}
\alias{html_document}
\title{Use Bioconductor style to format R Markdown HTML output}
\usage{
html_document(
  toc = TRUE,
  number_sections = TRUE,
  fig_width = NA,
  fig_height = NA,
  self_contained = TRUE,
  css = NULL,
  pandoc_args = NULL,
  ...,
  titlecaps = TRUE
)
}
\arguments{
\item{toc}{logical(1), \code{TRUE} to include a table of contents in the
output}

\item{number_sections}{logical(1), \code{TRUE} to number section headings}

\item{fig_width}{numeric(1), default width (in inches) for figures}

\item{fig_height}{numeric(1), default width (in inches) for figures}

\item{self_contained}{numeric(1), \code{TRUE} to produce a standalone HTML
file with no external dependencies, using data: URIs to incorporate the
contents of linked scripts, stylesheets, images, and videos. Note that even
for self contained documents MathJax is still loaded externally (this is
necessary because of it's size).}

\item{css}{character, one or more css files to include}

\item{pandoc_args}{character, additional command line options to pass to
pandoc}

\item{\dots}{Additional arguments passed to
\code{\link[rmarkdown]{html_document}}}

\item{titlecaps}{logical(1), \code{TRUE} to use the emphasize the first
sentence in figure and table captions as title}
}
\value{
R Markdown output format to pass to \code{\link[rmarkdown]{render}}
}
\description{
Format for converting from R Markdown to an Bioconductor HTML document.
}
\details{
\code{BiocStyle::html_document} format extends the
\code{\link[rmarkdown]{html_document}} format. See the
\href{http://rmarkdown.rstudio.com/html_document_format.html}{online
documentation} and the package vignette "Authoring R Markdown Vignettes" for
additional details on using the format,
}
\examples{

\dontrun{

# simple invocation
render("input.Rmd", BiocStyle::html_document())

# specify options
render("input.Rmd", BiocStyle::html_document(toc = FALSE))
}

}
\seealso{
\code{\link[BiocStyle]{pdf_document}},
\code{\link[BiocStyle]{md_document}}
}
\author{
Andrzej Oleś <andrzej.oles@embl.de>, 2014-2017
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/latex.R
\name{latex}
\alias{latex}
\title{Use Bioconductor style to format LaTeX vignettes}
\usage{
latex(
  ...,
  width,
  titlecaps = TRUE,
  short.fignames = FALSE,
  fig.path,
  use.unsrturl = TRUE,
  relative.path = FALSE
)
}
\arguments{
\item{\dots}{Additional arguments, passed to \code{\link{options}}.}

\item{width}{integer(1), maximum number of columns on a line used in
printing. See \code{\link{options}}.}

\item{titlecaps}{logical(1), emphasize the first sentence of float captions}

\item{short.fignames}{logical(1), indicates whether \code{\\incfig} figures
should be inserted and referred to using short names equal to corresponding
code chunk labels without any additional prefix.}

\item{fig.path}{character(1), custom prefix to be used for figure file names
when used with \code{knitr}; has no effect when compiled with \code{Sweave}.
For details see \code{\link[knitr]{opts_chunk}}.}

\item{use.unsrturl}{logical(1), indicating that the \sQuote{unsrturl} style
will be used (\code{\\bibliographystyle} command \emph{not} required).}

\item{relative.path}{logical(1), copy supporting LaTeX files and use
relative paths rather than absolute paths to system files.}
}
\value{
The return value is a character string directing LaTex to use the
Bioconductor style file.

A side effect is setting any options specified as arguments.
}
\description{
This function inserts code into a document preamble to provide a consistent
style to Bioconductor vignettes.
}
\details{
Use is described in the \sQuote{Bioconductor LaTeX Style 2.0} vignette.

By default the 'unsrturl' bibliography style is used, which automatically
creates links to URL, DOIs, etc. Use a different \code{bibliographystyle}
with \code{use.unsrturl=FALSE} and standard LaTeX commands for styling
bibliographies.
}
\examples{

## location of the .sty file
BiocStyle:::bioconductor.sty

}
\author{
Andrzej Oleś, Martin Morgan, Wolfgang Huber
}
\keyword{manip}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/markdown.R
\name{helpers}
\alias{helpers}
\alias{pkg_ver}
\alias{doc_date}
\title{Specify Rmarkdown document metadata}
\usage{
pkg_ver(pkg)

doc_date()
}
\arguments{
\item{pkg}{character(1), package name}
}
\value{
Markdown-formatted character string.
}
\description{
Helper functions for including metadata in the document header.
}
\details{
Use \code{doc_date} to include document compilation date in the document
metadata field 'date', and \code{pkg_ver} for package version specification
in the 'package' field.
}
\examples{


## current date
doc_date()

## package name with version
pkg_ver("BiocStyle")

}
\author{
Andrzej Oleś <andrzej.oles@embl.de>, 2014-2015
}
\keyword{manip}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/markdown.R
\name{output}
\alias{output}
\title{Output format of an R Markdown document}
\usage{
output()
}
\value{
A character string specifying the pandoc output format.
}
\description{
Helper function to determine the document's current pandoc output format.
}
\details{
The function is useful for defining different behavior depending on the
output format, e.g. figure settings.
}
\examples{

\dontrun{
## Switch between SVG and PDF figures depending on document output format
knitr::opts_chunk$set(
  dev = switch(output(), html = "svg", latex = "pdf")
  )
}

}
\author{
Andrzej Oleś <andrzej.oles@embl.de>, 2016
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pdf_document.R
\name{pdf_document}
\alias{pdf_document}
\title{Use Bioconductor style to format R Markdown PDF output}
\usage{
pdf_document(
  toc = TRUE,
  number_sections = TRUE,
  fig_width = NA,
  fig_height = NA,
  includes = NULL,
  ...,
  titlecaps = TRUE,
  toc_newpage = FALSE,
  use_unsrturl = TRUE,
  relative_path = FALSE
)
}
\arguments{
\item{toc}{logical(1), \code{TRUE} to include a table of contents in the
output}

\item{number_sections}{logical(1), \code{TRUE} to number section headings}

\item{fig_width}{numeric(1), default width (in inches) for figures}

\item{fig_height}{numeric(1), default width (in inches) for figures}

\item{includes}{Named list of additional content to include within the
document (typically created using the \code{\link{includes}} function).}

\item{\dots}{Additional arguments passed to
\code{\link[rmarkdown]{pdf_document}}.}

\item{titlecaps}{logical(1), \code{TRUE} to use the emphasize the first
sentence in figure and table captions as title}

\item{toc_newpage}{logical(1), \code{TRUE} to start the table of contents on
a new page.}

\item{use_unsrturl}{logical(1), indicating that the \sQuote{unsrturl} style
will be used (\code{\\bibliographystyle} command \emph{not} required).}

\item{relative_path}{logical(1), copy supporting LaTeX files and use
relative paths rather than absolute paths to system files.}
}
\value{
R Markdown output format to pass to \code{\link[rmarkdown]{render}}.
}
\description{
This function sets the Bioconductor style in PDF documents rendered using R
Markdown v2.
}
\examples{

\dontrun{

# simple invocation
render("input.Rmd", BiocStyle::pdf_document())

# specify an option for latex engine
render("input.Rmd", BiocStyle::pdf_document(toc = FALSE))
}

}
\seealso{
\code{\link[BiocStyle]{html_document}},
\code{\link[BiocStyle]{md_document}}
}
\author{
Andrzej Oleś <andrzej.oles@embl.de>, 2014-2017
}
\keyword{manip}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/markdown.R
\name{markdown}
\alias{markdown}
\title{Use Bioconductor CSS style to format HTML vignettes}
\usage{
markdown(css.files, self.contained = TRUE, links.target = TRUE)
}
\arguments{
\item{css.files}{character vector containing the location of additional
\code{.css} files.}

\item{self.contained}{logical(1), should the content of the CSS
\code{stylesheet} files be included into the html file or should they be
saved as separate files.}

\item{links.target}{logical(1), should external links open in new browser
tab/window.}
}
\value{
No value is returned. The function is called for its side effect of
setting the \code{markdown} and/or \code{knitr} specific options controlling
the inclusion of the Bioconductor CSS style file in the HTML output.
}
\description{
This function sets the Bioconductor style sheet to provide a consistent
style across Bioconductor HTML vignettes.
}
\details{
Use is described in the \sQuote{Bioconductor CSS Style} vignette.
}
\examples{

## location of the .css file
BiocStyle:::bioconductor.css

}
\author{
Andrzej Oleś <andrzej.oles@embl.de>, 2014-2015
}
\keyword{manip}
